package jagafa;

import jagafa.flags.TestingFlags;
import jagafa.scores.ScoreTable;
import jagafa.util.dump.Dump;
import jagafa.util.view.AbsConstr;
import jagafa.util.view.TimedDialog;
import jagafa.view.newui.RoundController2;
import jagafa.view.newui.TestingFlagsPanel;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;
import javax.swing.Timer;

/**
 * GameController skeleton
 *
 */
public class GameController implements Observer, ActionListener {
	private ScoreTable scoreTable_;

	private JFrame scoreTableFrame_;

	private JFrame roundFrame_;

	private JFrame ruleChoiceFrame_;

	private RoundController2 roundController_;

	private Timer nextRoundTimer_;

	private JInternalFrame rcFrame;

	public GameController() {
		roundController_ = new RoundController2();

	}

	public void startGame(ScoreTable scoreTable) {
		this.scoreTable_ = scoreTable;

		startRoundController();

		createRoundView();
		createRuleChoiceView();

		createScoreTableView();
		
		this.roundController_.toggleNoify();
		showHelp();
		showDump();
		showFlags();
	}

	private void createRoundView() {
		roundFrame_ = new JFrame();
		roundFrame_.setJMenuBar(getMenuBar());
		roundFrame_.setLayout(new BorderLayout());
		roundFrame_.getContentPane().add(this.roundController_.getRoundPanel(),
				BorderLayout.CENTER);
		roundFrame_.setVisible(true);
		roundFrame_.pack();
		roundFrame_.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		roundFrame_.setTitle("Jass by Matthias Amacher - (c) 2005");
		//roundFrame_.setResizable(false);
		roundFrame_.setLocation(200, 0);
		this.roundController_.getRuleChoicePanel().setRoundUI(this.roundFrame_);
		
		roundFrame_.validate();

		
	}

	private JMenuBar getMenuBar() {
		JMenuBar menubar = new JMenuBar();
		JMenu menu = new JMenu("Game");

		JMenuItem dump = new JMenuItem("Show Dumper");
		JMenuItem flags = new JMenuItem("Show Control-Flags");
		JMenuItem help = new JMenuItem("Show Help");
		JMenuItem exit = new JMenuItem("Exit");

		help.addActionListener(this);
		exit.addActionListener(this);
		flags.addActionListener(this);
		dump.addActionListener(this);
		
		menu.add(dump);
		menu.add(flags);
		menu.add(new JSeparator());
		menu.add(help);
		menu.add(new JSeparator());
		menu.add(exit);

		menubar.add(menu);

		return menubar;
	}

	private void createRuleChoiceView() {
	/*	rcFrame = new JInternalFrame();
		rcFrame.add(this.roundController_.getRuleChoicePanel());
		//rcFrame.setUndecorated(true);
		rcFrame.setVisible(true);
		rcFrame.pack();
		//rcFrame.setAlwaysOnTop(true);
		rcFrame.setTitle("Rule Choice");
		rcFrame.setLocation(560, 190);
		rcFrame.setResizable(false);
		rcFrame.setLocation(560 + this.roundFrame_.getX(), 150 + this.roundFrame_.getY());
		*/
		AbsConstr abs = new AbsConstr(0,0,130,200);
		AbsConstr abs2 = new AbsConstr(0,200,130,100);
		
		this.roundController_.getRoundPanel().add(this.roundController_.getRuleChoicePanel(),abs);
		this.roundController_.getRoundPanel().add(this.roundController_.getSTPanel(),abs2);
		this.roundController_.getRoundPanel().validate();
		//this.roundFrame_.getContentPane().add(this.roundController_.getRuleChoicePanel(),abs);
	}

	private void createScoreTableView() {
		/*scoreTableFrame_ = new JFrame();
		scoreTableFrame_.setUndecorated(true);
		scoreTableFrame_.add(this.roundController_.getSTPanel());
		scoreTableFrame_.setVisible(true);
		scoreTableFrame_.pack();
		scoreTableFrame_.setAlwaysOnTop(true);
		scoreTableFrame_.setTitle("Score Table for SCHIEBER");
		scoreTableFrame_.setResizable(false);
		scoreTableFrame_.setEnabled(false);
		scoreTableFrame_.setLocation(560, 390);
		scoreTableFrame_.setSize(this.rcFrame.getWidth(), scoreTableFrame_.getHeight());
	*/
	}

	private boolean isGameOver() {
		return this.scoreTable_.isGameOver();
	}

	private void startRoundController() {
		this.roundController_.startRound(this.scoreTable_);
		this.roundController_.addObserver(this);
	}

	public void update(Observable arg0, Object arg1) {
		
		if (this.isGameOver()) {
			new TimedDialog("Game is over!\n" + this.scoreTable_.toString(),
				this.roundFrame_.getRootPane(), 5000);
		}
	}

	public void actionPerformed(ActionEvent arg0) {
		String command = arg0.getActionCommand();

		if (command.equals("Show Help")) {
			showHelp();
		} else if (command.equals("Exit")) {
			Runtime.getRuntime().exit(0);
		} else if (command.equals("Show Dumper")) {
			this.toggleDump();
		} else if (command.equals("Show Control-Flags")) {
			this.toggleFlags();
		}

	}

	private void toggleDump() {
		if (this.dumpFrame.isVisible()) {
			this.dumpFrame.setVisible(false);
		} else {
			this.dumpFrame.setVisible(true);
		}
	}
	private void toggleFlags() {
		if (this.flagFrame.isVisible()) {
			this.flagFrame.setVisible(false);
		} else {
			this.flagFrame.setVisible(true);
		}
	}

	private void showHelp() {
		String help1 = "1) To start the game, select a Trump and press the\n'Set Rule' button";
		String help2 = "\n2) If you want to 'schieben' then press the 'Hand choice over' button";
		String help3 = "\n3) If one round is over press 'Deal' and the AI will choose a RuleSet";
		String help4 = "\n4) The current Rule Set will be displayed on the Rule Choice panel on the left";
		String help5 = "\n\n5) The small panel on the left displays the current score of both teams.\n\tTeam1 is" 
			+ " are the Players on the top and bottom.\n\tTeam2 are the Players on the left and right.";
		new TimedDialog(help1 + help2 + help3 + help4+help5, roundFrame_.getRootPane(), 8000);

	}

	private void showDump() {
		TestingFlags.redirectSystemOut_ = true;
		TestingFlags.ruleSetChoiceDump_ = true;
		Dump dump = new Dump();
		dumpFrame = new JFrame();
		dumpFrame.add(Dump.getDumpPanel());
		dumpFrame.setUndecorated(true);
		dumpFrame.setVisible(false);

		dumpFrame.setPreferredSize(new Dimension(200, 300));
		dumpFrame.pack();
	}

	private JFrame dumpFrame;

	private JFrame flagFrame;

	private void showFlags() {

		flagFrame = new JFrame();
		flagFrame.add(new TestingFlagsPanel());
		flagFrame.setUndecorated(true);
		flagFrame.setVisible(false);

		flagFrame.setPreferredSize(new Dimension(200, 300));
		flagFrame.setLocation(0, 300);
		flagFrame.pack();
	}
}
