/**
 * 
 */
package jagafa;

import jagafa.object.Board;
import jagafa.object.Card;
import jagafa.object.Distributor;
import jagafa.object.GoneCardHeap;
import jagafa.object.Player;
import jagafa.rule.RuleSet;
import jagafa.scores.ScoreTable;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Observable;

import javax.swing.Timer;
import javax.swing.event.EventListenerList;

/**
 * @author Besitzer
 * 
 */
public abstract class JassRoundCore extends Observable {

	/* TODO: Externalize FINAL_ROUND_SCORE and LAST_TURN_SCORE */
	private static final int LAST_TURN_SCORE = 5;

	private static final int FINAL_ROUND_SCORE = 152;

	private static final int PLAYERS = 4;

	private static final String UPDATED_EVENT = "UPDATED";

	private static final int BOARD_SIZE = 4;

	private EventListenerList listeners_;

	private RuleSet rules_;

	private Board activeBoard_;

	private List<Player> players_;

	private Player activePlayer_;

	private Iterator<Player> playerIterator_;

	private Player lastTurnWinner_;

	private GoneCardHeap goneHeap_;

	protected boolean running_ = false;

	protected Timer refreshTimer_;

	protected Timer aiTimer_;

	protected ScoreTable scoreTable_;

	/**
	 * Initialize a new game using the RuleSet given
	 */
	public JassRoundCore(RuleSet rules) {
		reset();
		this.setRules(rules);
		

		this.toggleNotify();
	}

	/**
	 * Set the RuleSet
	 */
	public void setRules(RuleSet r) {
		this.rules_ = r;
		this.sortHands();
	}

	public boolean isRunning() {
		return this.running_;
	}

	/**
	 * Returns the active Players' id
	 */
	public int activePlayerNr() {
		for (int i = 0; i < PLAYERS; i++) {
			Player p = this.players_.get(i);
			if (p == activePlayer()) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * Returns the active Player
	 */
	public Player activePlayer() {
		return this.activePlayer_;
	}

	/**
	 * Counts the score on a board
	 */
	protected int getScore(Board board) {
		int sum = 0;
		for (int i = 0; i < BOARD_SIZE; i++) {
			sum += rules_.getScore(board.get(i));
		}

		return sum;
	}

	/**
	 * Returns the winning Player of the board given
	 */
	protected Player getWinner(Board board) {
		for (int i = 0; i < BOARD_SIZE; i++) {
			Card c = board.get(i);
			if (rules_.compareHigher(c, board)) {

				int startPlayer = Integer.valueOf(lastTurnWinner_.getName()).intValue();
				int winner = startPlayer + i;
				if (winner >= PLAYERS) {
					winner -= PLAYERS;
				}

				return this.getPlayer(winner);
			}
		}
		return null;
	}

	/**
	 * Returns true if the game is over
	 */
	public boolean isRoundOver() {
		int sum1 = this.getPlayer(0).getScore() + this.getPlayer(2).getScore();
		int sum2 = this.getPlayer(1).getScore() + this.getPlayer(3).getScore();
		if (sum1 + sum2 == FINAL_ROUND_SCORE) {
			return true;
		}
		if (sum1 + sum2 == FINAL_ROUND_SCORE + LAST_TURN_SCORE) {
			return true;
		}

		return false;

	}

	/**
	 * Sets the active player to the next
	 */
	protected Player nextPlayer() {
		if (!this.playerIterator_.hasNext()) {
			this.playerIterator_ = this.players_.iterator();
		}
		this.activePlayer_ = playerIterator_.next();
		return activePlayer();
	}

	/**
	 * Set the active player
	 */
	public void setActivePlayer(Player playerToSet) {
		while (this.activePlayer() != playerToSet) {
			this.nextPlayer();
		}
	}

	/** 
	 * Notify observers and fire all action-listeners 
	 */
	public void toggleNotify() {
		this.setChanged();
		this.notifyObservers();
		this.clearChanged();
		this.fireAction();
	}
/*
	public void addActionListener(ActionListener l) {
		this.listeners_.add(ActionListener.class, l);
	}
*/
	private void fireAction() {
		Object[] listeners = this.listeners_.getListenerList();
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == ActionListener.class) {
				ActionEvent fooEvent = new ActionEvent(this, 0, UPDATED_EVENT);
				((ActionListener) listeners[i + 1]).actionPerformed(fooEvent);
			}
		}

	}

	/**
	 * Returns the player with the id given 
	 */
	public Player getPlayer(int i) {
		while (i >= 4) {
			i -= 4;
		}
		return this.players_.get(i);
	}

	/**
	 * Returns the Player which was the winner of the previous turn
	 */
	protected Player getLastTurnWinner() {
		return this.lastTurnWinner_;
	}

	/**
	 * Returns an id for the player p
	 */
	public int getPlayerNr(Player p) {
		for (int i = 0; i < PLAYERS; i++) {
			if (this.getPlayer(i) == p) {
				return i;
			}

		}
		return -1;

	}

	/**
	 * Returns the RuleSet
	 */
	public RuleSet getRules() {
		return this.rules_;
	}

	/**
	 * Returns the Gone Card Heap
	 */
	public GoneCardHeap getGoneHeap() {
		return this.goneHeap_;
	}

	/**
	 * Returns the Partner of Player pl
	 */
	public Player getPartner(Player pl) {
		int nr = this.getPlayerNr(pl);
		nr += 2;
		if (nr >= 4) {
			nr -= PLAYERS;
		}
		
		return this.getPlayer(nr);
	}

	/**
	 * Returns the opponents of the Player p
	 */
	public Player[] getOpponents(Player p) {
		Player opp[] = new Player[2];

		int nr = this.getPlayerNr(p);
		
		nr += 1;
		if (nr > 3) {
			nr -= PLAYERS;
		}
		opp[0] = this.getPlayer(nr);
		nr = this.getPlayerNr(opp[0]) + 2;
		if (nr > 3) {
			nr -= PLAYERS;
		}
		opp[1] = this.getPlayer(nr);
		return opp;

	}

	/**
	 * Checks wether a card is valid to play on the active Board or not
	 */
	public boolean isValidPlay(Card toPlay) {
		if (refreshTimer_==null) {
			return false;
		}
		
		if (!refreshTimer_.isRunning()) {
			return this.rules_.isValid(toPlay, activePlayer().getHand(),
					this.activeBoard_);
		} else {

			return false;

		}
	}

	/**
	 * Returns the active Board
	 */
	public Board activeBoard() {
		return this.activeBoard_;
	}

	/**
	 * Reset everything
	 */
	public void reset() {
		this.running_ = false;

		this.players_ = new LinkedList<Player>();
		this.listeners_ = new EventListenerList();
		this.activeBoard_ = new Board();
		this.goneHeap_ = new GoneCardHeap();

		initPlayers();

		distributeCards();

		this.playerIterator_ = players_.iterator();
		this.activePlayer_ = this.playerIterator_.next();
		this.lastTurnWinner_ = this.activePlayer_;
		this.toggleNotify();
		
		
		

	}

	
	/**
	 * Distribute the cards to the four players
	 */
	private void distributeCards() {
		Distributor.distributeCardsTo(this.players_);
		sortHands();
	}

	/**
	 * Sort the players hands (only if the RuleSet is set)
	 *
	 */
	private void sortHands() {
		if (this.getRules() == null) {
			return;
		}
		for (int i = 0; i < PLAYERS; i++) {
			Player p = this.getPlayer(i);
			p.getHand().sort(this.getRules());
		}
	}

	protected abstract void initPlayers();

	protected void setLastTurnWinner(Player winner) {
		this.lastTurnWinner_ = winner;

	}

	protected void addPlayer(Player newPlayer) {
		this.players_.add(newPlayer);
	}

	/** 
	 * Sets the score table
	 * @param table The relevant score table where the rounds score will be reported
	 */
	public void setScoreTable(ScoreTable table) {
		this.scoreTable_ = table;

	}

	/** 
	 * Get the score table
	 * @return The score table relevant for this round
	 */
	public ScoreTable getScoreTable() {
		return this.scoreTable_;
	}

	/** 
	 * Update the score table after a round is finished
	 * (Report round scores to top-level game)
	 */
	protected void reportScores() {
		if (!this.isRoundOver()) {
			return;
		}
		if (getScoreTable() != null) {
			int scores[] = new int[4];
			for (int i = 0; i < 4; i++) {
				scores[i] = this.getPlayer(i).getScore();
			}

			this.getScoreTable().addScores(scores, this.getRules());
			
			/*
			if (this.getScoreTable().isGameOver()) {
				int t1 = this.getScoreTable().getScores()[0]
						+ this.getScoreTable().getScores()[2];
				int t2 = this.getScoreTable().getScores()[1]
						+ this.getScoreTable().getScores()[3];
				System.out.println("Schieber is over! " + t1 + "  " + t2);
				this.running_ = false;
			}*/
		}
	}

	public void stop() {
		this.running_ = false;
		if (this.aiTimer_ != null && this.refreshTimer_!=null) {
			this.aiTimer_.stop();
			this.refreshTimer_.stop();
		}

	}
}
