/**
 * 
 */
package jagafa.ai;

import jagafa.JassRound;
import jagafa.ai.core.JassAI;
import jagafa.ai.core.RuleSetChoiceAI;
import jagafa.flags.TestingFlags;
import jagafa.object.Board;
import jagafa.object.Card;
import jagafa.object.CardList;
import jagafa.object.GoneCardHeap;
import jagafa.object.Hand;
import jagafa.object.Player;
import jagafa.rule.RuleSet;
import jagafa.stats.PlayerStats;

/**
 * A simple (quite stupid) AI implementation
 * 
 */
public class SimpleAI implements JassAI {

	private Player player_;

	private JassRound round_;

	private RuleSet rules_;

	private GoneCardHeap goneHeap_;

	private Hand hand_;

	private int trumpfColor_;

	private Board board_;

	private PlayerStats stats_;
	private int turnNr = -1;
	
	
	public Card computeCard() {
		this.hand_ = this.player_.getHand();
		this.stats_ = new PlayerStats(this.round_,this.player_);
		this.goneHeap_ = this.round_.getGoneHeap();
		Card c = getCard();
		if (TestingFlags.aiStrategyDump_)System.out.println(c);
		return c;

	}

	
	/**
	 * @return
	 */
	private Card getCard() {
		
		this.stats_.update();
		//this.stats_.printTree();
		this.hand_ = this.player_.getHand();

		if (this.round_.activeBoard().get(0) != null) {
			Card firstCard = board_.get(0);
			
			CardList cardsOfFirst = stats_.getColorList()[firstCard.getColor()];

			if (cardsOfFirst.size() == 0) {
				if (TestingFlags.aiStrategyDump_)System.out.print("Throw: ");
				return throwAway(); // Verrr
			} else {
				if (TestingFlags.aiStrategyDump_)System.out.print("Color: ");
				return sameColor(firstCard.getColor()); // Farben spielen
			}
		} else {
			if (hasBock(0)) {
				if (TestingFlags.aiStrategyDump_)System.out.print("Bock: ");
				return getBock(); // Bcke spielen
			}
			if (TestingFlags.aiStrategyDump_)System.out.print("Tear: ");
			return tearOut(); // Farbe Anziehen bzw. dem Partner bringen
		}
		
	}

	/**
	 * @return
	 */
	private Card tearOut() {
		int colorWithMost = getColorWithMost(stats_.getColorList());

		for (int bockNth = 8; bockNth > 5; bockNth--) {
			if (hasBock(bockNth, colorWithMost)) {
				return getBock(bockNth, colorWithMost);
			}
		}

		for (int bockNth = 8; bockNth > 0; bockNth--) {
			if (hasBock(bockNth)) {
				return getBock(bockNth);
			}
		}
		return null;
	}

	/**
	 * @return
	 */
	private Card getBock() {
		Card resultCard = null;
		int colorWithMost = getColorWithMost(stats_.getColorList());
		if (colorWithMost != -1) {
			resultCard = getBock(0, colorWithMost); 
			return resultCard;
		}
		int randomNr = (int) (Math.random() * 4);
		resultCard = stats_.getBockList()[randomNr].get(0);
		return resultCard;
	}

	private Card getBock(int nth) {
		CardList bockI = stats_.getBockList()[nth];
		if (bockI.size() > 0) {
			for (int i = 0; i < bockI.size(); i++) {

				return bockI.get(i);

			}

		}
		return null;
	}

	// Gibt einen bock der farbe color zurck
	private Card getBock(int nth, int color) {
		Card resultCard = null;
		CardList bockI = stats_.getBockList()[nth];
		if (bockI.size() > 0) {
			for (int i = 0; i < bockI.size(); i++) {
				if (bockI.get(i).getColor() == color) {
					return bockI.get(i);
				}
			}

		}
		return resultCard;
	}

	/**
	 * @return
	 */
	private int getColorWithMost(CardList cardL[]) {
		int mostNrColor = -1;
		int colorWithMost = -1;

		for (int i = 0; i < cardL.length; i++) {
			CardList colori = cardL[i];
			int nrColorI = colori.size();
			if (nrColorI >= mostNrColor && hasBock(0, i)) {
				mostNrColor = nrColorI;
				colorWithMost = i;

			}
		}
		return colorWithMost;
	}

	/**
	 * @return
	 */
	private boolean hasBock(int nth) {
		for (int i = 0; i < 4; i++) {
			if (hasBock(nth, i)) {
				return true;
			}

		}
		return false;
	}

	private boolean hasBock(int nth, int color) {
		CardList bockI = stats_.getBockList()[nth];
		if (bockI.size() > 0) {
			for (int i = 0; i < bockI.size(); i++) {
				if (bockI.get(i).getColor() == color) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * @param cardsOfFirst
	 */
	private Card sameColor(int color) {
		if (hasBock(0, color)) {
			return getBock(0, color);
		} else {
			for (int bockNth = 8; bockNth > 0; bockNth--) {
				if (hasBock(bockNth, color)) {
					return getBock(bockNth, color);
				}
			}
		}
		return null;
	}

	/**
	 * 
	 */
	private Card throwAway() {
		int throwAwayColor = -1;
		int minColorValue = -1;
		for (int c = 0; c < 4; c++) {
			int colorValue = getColorValue(c);
			if (colorValue > minColorValue) {
				minColorValue = colorValue;
				throwAwayColor = c;
			}
		}

		for (int bockNth = 8; bockNth >= 0; bockNth--) {
			if (hasBock(bockNth, throwAwayColor)) {
				return getBock(bockNth, throwAwayColor);
			}
		}
		for (int c = 0; c < 4; c++) {
			for (int bockNth = 8; bockNth >= 0; bockNth--) {
				if (hasBock(bockNth, c)) {
					return getBock(bockNth, c);
				}
			}
		}
		return null;

	}

	/**
	 * @param c
	 * @return
	 */
	private int getColorValue(int c) {
		int value = 0;
		for (int i = 0; i < 8; i++) {
			if (hasBock(i, c)) {
				value += (i) * (i);
			}
		}
		return value;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jass.JassAI#init(jass.JassGame)
	 */
	public void init(Player p, JassRound round) {
		this.player_ = p;
		this.round_ = round;
		this.rules_ = round.getRules();
		this.goneHeap_ = round.getGoneHeap();
		this.board_ = round_.activeBoard();

	}


	public RuleSet chooseRuleSet() {
		return RuleSetChoiceAI.chooseRuleSet(this.player_,this.round_);
	}

}
