package jagafa.ai;

import jagafa.JassRound;
import jagafa.ai.core.JassAI;
import jagafa.ai.core.ProbabilityTable;
import jagafa.ai.core.RuleSetChoiceAI;
import jagafa.flags.TestingFlags;
import jagafa.naming.ESwissColorNames;
import jagafa.object.Board;
import jagafa.object.Card;
import jagafa.object.CardList;
import jagafa.object.Player;
import jagafa.rule.RuleSet;
import jagafa.stats.PlayerStats;

/**
 * AI - For Trumpf and Kij RuleSets
 * Uses the ProbabilityTable for the Throw/Bring strategy only
 * 
 */
public class TrumpfProbAI implements JassAI {

	private static final int FIRST_CARD = 0;

	private static final double COLORS = 4;

	private static final int PLAYERS = 4;

	private Player player_;

	private JassRound round_;

	private RuleSet rules_;

	private int trumpfColor_;

	private Board board_;

	private ProbabilityTable probTable_;

	private PlayerStats stats_;

	
	public Card computeCard() {
		this.stats_ = new PlayerStats(this.round_, this.player_);
		this.probTable_ = new ProbabilityTable(this.player_, this.round_);
		Card c = getCard();

		if (TestingFlags.aiStrategyDump_)
			if (c != null) {
				System.out.print("\t" + ESwissColorNames.getName(c.getColor()));
				System.out.println("    \t" + this.round_.getRules().getPower(c,this.board_));
			}
		return c;
	}

	/**
	 * Computes a card according to the strategy
	 */
	private Card getCard() {
		if (TestingFlags.aiStrategyDump_) {
			System.out.println("Player\t" + this.player_.getName() + "...");
		}

		this.stats_.update();
		
		if (this.round_.activeBoard().cardsPlayed()>0) {
			Card firstCard = board_.get(FIRST_CARD);

			CardList cardsOfFirst = stats_.getColorList()[firstCard.getColor()];

			if (cardsOfFirst.size() == 0) {
				if (TestingFlags.aiStrategyDump_)
					System.out.print(" Throw: ");
				
				return throwAway(); // Verrr oder Trumpf
			} else {
				if (TestingFlags.aiStrategyDump_)
					System.out.print(" Color: ");
				// Farben oder Trumpf
				return sameColor(firstCard.getColor()); // Farben spielen
			}
		} else {
			// Austrumpfen oder Bcke spielen oder Ziehen/Bringen
			
			
			if (hasBock(0)) {
				if (TestingFlags.aiStrategyDump_)
					System.out.print(" Bock:   ");
				return getBock(); // Bcke spielen
			}

			return tearOut(); // Farbe Anziehen bzw. dem Partner bringen
		}

	}

	/**
	 * This strategy is played 
	 * 1) if the player doesn't have any Card of firstcolor or
	 * 2) If he brings a color to his partner.
	 * In the second case the color with the highest partner-probability is played
	 */
	private Card tearOut() {
		double highestProb = -100;
		int actHProb = -100;
		int secondHProb = -100;
		Player opp[] = round_.getOpponents(this.player_);

		Player partner = round_.getPartner(this.player_);
		for (int col = 0; col < COLORS; col++) {
			float oppProb1 = probTable_.getColorProb(opp[0], col);
			float oppProb2 = probTable_.getColorProb(opp[1], col);
			float oppProb = (oppProb1 + oppProb2) / 8;
			float prob = probTable_.getColorProb(partner, col) - oppProb;
			if (TestingFlags.probAIDump_) {
				System.out.println(" \tProb\t" + ESwissColorNames.values()[col] + "= "
						+ prob);
			}
			if (prob > highestProb) {
				highestProb = prob;
				secondHProb = actHProb;
				actHProb = col;
			}

		}

		for (int bockNth = 8; bockNth > 0; bockNth--) {
			if (hasBock(bockNth, actHProb)) {
				Card b = getBock(bockNth, actHProb);
				if (TestingFlags.aiStrategyDump_)
					if (TestingFlags.aiStrategyDump_) {
						System.out.println("Using Probtable:\n");
						System.out.println(probTable_.toString());

						System.out.print(" Bring first: ");
					}
				return b;
			}
		}

		if (secondHProb != -100) {
			for (int bockNth = 8; bockNth > 0; bockNth--) {
				if (hasBock(bockNth, secondHProb)) {
					Card b = getBock(bockNth, secondHProb);
					if (TestingFlags.aiStrategyDump_) {
						System.out.println("Using Probtable:\n");
						System.out.println(probTable_.toString());

						System.out.print(" Bring second: ");
					}
					return b;
				}
			}
		}

		for (int bockNth = 8; bockNth > 0; bockNth--) {
			if (hasBock(bockNth)) {
				if (TestingFlags.aiStrategyDump_)
					System.out.print(" Tear: ");
				return getBock(bockNth);
			}
		}
		return null;
	}

	/**
	 * Gibt einen 0. Bock (wenn vorhanden, sonst zuflliger) der Farbe mit den 
	 * meisten Karten zurck  
	 */
	private Card getBock() {
		Card resultCard = null;
		int colorWithMost = getColorWithMost(stats_.getColorList());
		if (colorWithMost != -1) {
			resultCard = getBock(0, colorWithMost); // stats_.getBockList()[colorWithMost].get(0);
			return resultCard;
		}
		int randomNr = (int) (Math.random() * COLORS);
		resultCard = stats_.getBockList()[randomNr].get(0);
		System.out.print("Random ");
		return resultCard;
	}

	/** 
	 * Gibt einen n-ten Bock beliebiger Farbe zurck 
	 */
	private Card getBock(int nth) {
		CardList bockI = stats_.getBockList()[nth];
		if (bockI.size() > 0) {
			for (int i = 0; i < bockI.size(); i++) {

				return bockI.get(i);

			}

		}
		return null;
	}

	/**
	 * Gibt einen n-ten Bock der Farbe color zurck
	 */
	private Card getBock(int nth, int color) {
		Card resultCard = null;
		CardList bockI = stats_.getBockList()[nth];
		if (bockI.size() > 0) {
			for (int i = 0; i < bockI.size(); i++) {
				if (bockI.get(i).getColor() == color) {
					return bockI.get(i);
				}
			}

		}
		return resultCard;
	}

	/**
	 * @return
	 */
	private int getColorWithMost(CardList cardL[]) {
		int mostNrColor = -1;
		int colorWithMost = -1;

		for (int i = 0; i < cardL.length; i++) {
			CardList colori = cardL[i];
			int nrColorI = colori.size();
			if (nrColorI >= mostNrColor && hasBock(0, i)) {
				mostNrColor = nrColorI;
				colorWithMost = i;

			}
		}
		return colorWithMost;
	}

	/**
	 * Returns true if the player has a Bock of grade n(th)
	 */
	private boolean hasBock(int nth) {
		for (int i = 0; i < COLORS; i++) {
			if (hasBock(nth, i)) {
				return true;
			}

		}
		return false;
	}

	/**
	 * Returns true if the player has a Bock of grade n(th) and the Color given
	 */
	private boolean hasBock(int nth, int color) {
		CardList bockI = stats_.getBockList()[nth];
		if (bockI.size() > 0) {
			for (int i = 0; i < bockI.size(); i++) {
				if (bockI.get(i).getColor() == color) {
					return true;
				}
			}
		}
		return false;
	}

	private boolean isBock(int nth, Card c) {
		int color = c.getColor();
		CardList bockList = this.stats_.getBockList()[nth];
		if (bockList.contains(c)) {
			return true;
		}

		return false;

	}

	/**
	 * Play a card of the color specified
	 */
	private Card sameColor(int color) {
		if (this.round_.getGoneHeap().turns() == 0) {
			if (this.board_.size() == 2) {
				/* Spiele den hchsten der Farbe die Partner zuerst anspielt */
				for (int bockNth = 0; bockNth < 8; bockNth++) {
					if (hasBock(bockNth, color)) {
						return getBock(bockNth, color);
					}
				}

			}
		}
		/* Wenn letzer des Zuges, dann kann er auch mit einem niedrigeren Stechen, 
		 * falls vorhanden!
		 */
		if (this.board_.size()==3) {
			for (int bockNth = 8; bockNth > 0; bockNth--) {
				if (hasBock(bockNth, color)) {
					if (this.rules_.compareHigher(getBock(bockNth,color),this.board_)){
						return getBock(bockNth, color);	
					}
				}
			}
			
			
		}
		

		Card firstCard = this.board_.getFirstCard();
		if (hasBock(0, color)) {
			return getBock(0, color);
		} else {
			for (int bockNth = 8; bockNth > 0; bockNth--) {
				if (hasBock(bockNth, color)) {

					return getBock(bockNth, color);
				}
			}
		}
		return null;
	}

	/**
	 * Throw away or bring a Card to the Partner 
	 */
	private Card throwAway() {
		int throwAwayColor = -1;
		int maxColorValue = -1;

		for (int c = 0; c < COLORS; c++) {
			CardList colorList = this.stats_.getColorList()[c];

			if (colorList.size() == 1) {
				if (!isBock(0, colorList.get(0))) {
					return colorList.get(0);
				}

			}

			int colorValue = getColorValue(c);
			if (colorValue > maxColorValue) {
				maxColorValue = colorValue;
				throwAwayColor = c;
			}
		}

		for (int bockNth = 8; bockNth >= 0; bockNth--) {
			if (hasBock(bockNth, throwAwayColor)) {
				return getBock(bockNth, throwAwayColor);
			}
		}
		for (int c = 0; c < COLORS; c++) {
			for (int bockNth = 8; bockNth >= 0; bockNth--) {
				if (hasBock(bockNth, c)) {
					return getBock(bockNth, c);
				}
			}
		}
		return null;

	}

	/**
	 * @param c
	 * @return
	 */
	private int getColorValue(int c) {
		int value = 0;
		for (int i = 0; i < 9; i++) {
			if (hasBock(i, c)) {
				value += (i) * (i);
			}
		}
		return value;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jass.JassAI#init(jass.JassGame)
	 */
	public void init(Player p, JassRound game) {
		this.player_ = p;
		this.round_ = game;
		this.rules_ = game.getRules();
		this.board_ = round_.activeBoard();
	}

	public RuleSet chooseRuleSet() {
		return RuleSetChoiceAI.chooseRuleSet(this.player_, this.round_);
	}

}
