package jagafa.ai.core;

import jagafa.JassRound;
import jagafa.flags.AIFlags;
import jagafa.flags.TestingFlags;
import jagafa.object.CardList;
import jagafa.object.Hand;
import jagafa.object.Player;
import jagafa.rule.ObenRules;
import jagafa.rule.RuleSet;
import jagafa.rule.UntenRules;
import jagafa.scores.ScoreTable;
import jagafa.stats.CardTool;
import jagafa.stats.PlayerStats;

import java.util.List;

public abstract class RuleSetChoiceAI {

	private static final int COLORS = 4;

	private static Player player_;

	private static ScoreTable scoreTable_;

	private static JassRound round_;

	public static RuleSet chooseRuleSet(Player p, JassRound round) {

		player_ = p;
		scoreTable_ = round.getScoreTable();
		round_ = round;

		List<RuleSet> rules_ = scoreTable_.getRules();

		RuleSet rule = null;

		double estimates[] = new double[rules_.size()];
		Hand h = player_.getHand();
		if (TestingFlags.ruleSetChoiceDump_) {
			//System.out.println("Player's hand: " + h);
		}
		int highest = -10;
		double highestEstimate = -100;
		double gradeWeight = AIFlags.GRADE_WEIGHT;

		for (int i = 0; i < rules_.size(); i++) {

			estimates[i] = gradeWeight * gradeRuleSet(rules_.get(i));
			if (rules_.get(i).getName().startsWith("Slalom")) {
				estimates[i] = gradeSlalom(rules_.get(i));
			}

			//estimates[i] += (1 - gradeWeight) * estimateScore(rules_.get(i));
			if (estimates[i] > highestEstimate) {
				highest = i;
				highestEstimate = estimates[i];
			}
		}

		if (TestingFlags.ruleSetChoiceDump_) {
			System.out.println("\nStrength estimates: ");
			for (int i = 0; i < rules_.size(); i++) {
				System.out.println(rules_.get(i).getName() + "     \t" + estimates[i]);

			}
		}
		if (highestEstimate < AIFlags.CHANGE_TRESHOLD) {

			if (scoreTable_.canChangeChoosingPlayer()) {
				return null;
			}
		}

		rule = rules_.get(highest);

		return rule;

	}

	private static double gradeSlalom(RuleSet set) {
		ObenRules oben = new ObenRules();
		UntenRules unten = new UntenRules();

		double obenEst = gradeRuleSet(oben);
		double untenEst = gradeRuleSet(unten);

		JassRound round = new JassRound(oben);

		int secureOben = 0;
		int addOben = 0;
		PlayerStats stats_ = new PlayerStats(round, player_);
		CardList bockList[] = stats_.getBockList();

		for (int i = 0; i < COLORS; i++) {
			secureOben += bocksOfColorInRow(bockList, i);
			CardList colorList = stats_.getColorList()[i];

			int addOfColor = (colorList.size() - secureOben);
			if (addOfColor > 2) {
				addOben += addOfColor;
			}

		}

		round = new JassRound(oben);

		int secureUnten = 0;
		int addUnten = 0;
		stats_ = new PlayerStats(round, player_);
		bockList = stats_.getBockList();

		for (int i = 0; i < COLORS; i++) {
			secureUnten += bocksOfColorInRow(bockList, i);

			CardList colorList = stats_.getColorList()[i];

			int addOfColor = (colorList.size() - secureUnten);
			if (addOfColor > 2) {
				addUnten += addOfColor;
			}

		}

		int diff = Math.abs(secureOben - secureUnten);
		double xxx = 1/(diff+0.5);
		
		return xxx*(secureOben+secureUnten);
	}

	public static double gradeRuleSet(RuleSet rule) {
		double scoreEstimate = 0;

		Hand h = player_.getHand();

		round_.setRules(rule);
		PlayerStats stats_ = new PlayerStats(round_, player_);

		CardList bockList[] = stats_.getBockList();
		for (int i = 0; i < bockList.length; i++) {
			CardList nthBock = bockList[i];
			int nrOfNth = nthBock.size();

			scoreEstimate += (double) ((1 / ((double) i + 1)) * (double) nrOfNth);
			//System.out.print(nrOfNth + ";"+ scoreEstimate + " " );
		}

		int numberOfColors = 0;

		for (int i = 0; i < COLORS; i++) {
			CardList ofCol = CardTool.getAllOfColor(h, i);

			int nrOfCol = ofCol.size();
			if (nrOfCol > 0) {
				numberOfColors++;
			}
			scoreEstimate += (double) (nrOfCol - 2 > 0 ? nrOfCol - 2 : 1)
					* (double) bocksOfColorInRow(bockList, i);
		}
		scoreEstimate += numberOfColors;
		return (double) (Math.round(scoreEstimate * 1000.0) / 1000.0);

	}

	private static int bocksOfColorInRow(CardList bockList[], int color) {
		int inarow = 0;
		boolean fail = false;
		for (int i = 0; i < bockList.length; i++) {
			CardList nthBock = bockList[i];
			CardList bockColor = CardTool.getAllOfColor(nthBock, color);

			if (bockColor.size() == 0) {
				/*if (fail) {
				 return inarow;
				 }
				 fail = true;
				 inarow++;*/
				return inarow;
			} else {
				inarow++;
			}

		}

		return inarow;
	}

	public static int estimateScore(RuleSet rule) {
		int strikesEstimate = 0;

		Hand h = player_.getHand();

		round_.setRules(rule);
		PlayerStats stats_ = new PlayerStats(round_, player_);

		CardList bockList[] = stats_.getBockList();

		for (int i = 0; i < bockList.length; i++) {
			CardList nthBock = bockList[i];
			int nrOfNth = nthBock.size();

		}
		int directStrikes = 0;
		int additionalEstimate = 0;
		for (int i = 0; i < COLORS; i++) {
			int secureStrikes = bocksOfColorInRow(bockList, i);
			directStrikes += secureStrikes;
			CardList colorList = stats_.getColorList()[i];

			int addOfColor = (colorList.size() - secureStrikes);
			if (addOfColor > 2) {
				additionalEstimate += addOfColor;
			}

		}

		double directWeight = 22.0;
		double addWeight = 10.0;
		double mult = addWeight / directWeight;

		strikesEstimate = directStrikes + (int) Math.round(mult * additionalEstimate);

		double scoreEst = directStrikes * directWeight + additionalEstimate * addWeight;

		/*
		 System.out.print("For " + rule.getName().toUpperCase() + "\t");
		 System.out.println("Estimated Score: " + scoreEst);
		 System.out.println("Where:\tdirect: " + directStrikes + "\tadditional: " + additionalEstimate);
		 System.out.println("Weighted:\t" + strikesEstimate + "\n");
		 */

		return strikesEstimate;
	}
}
