/*
 * Created on 27.03.2005 Filename: CardHolder.java
 */
package jagafa.object;

import jagafa.naming.ESwissColorNames;
import jagafa.naming.ESwissValueNames;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Object to store a list of Card Objects
 * Attention: Does not implement java.util.List
 */
public class CardList{
    private List<Card> cards_;

    /**
     * Constructor: Init the list
     */
    public CardList() {
        this.cards_ = new LinkedList<Card>();
    }

    /**
     * Add a card to the CardList
     * @param c The card to add
     */
    public void add(Card c) {
        this.cards_.add(c);
    }

    /**
     * Remove a card from the CardList
     * @param c The card to remove
     */
    public void remove(Card c) {
        this.cards_.remove(c);
    }

    /**
     * Remove a card from the CardList
     * @param i The cards position
     */
    public void remove(int i) {
        this.cards_.remove(i);
    }

    /**
     * Get a card out of the CardList at a position specified
     * @return The card at the ith position in the CardList
     * @param i The position
     */
    public Card get(int i) {
        if (i >= this.size() || i <0) {
            return null;
        }
        return this.cards_.get(i);

    }

    /**
     * Converts the CardList to a java.util.iterator object
     * @return An iterator of Card objects 
     */
    public Iterator<Card> iterator() {
        return this.cards_.iterator();
    }

    /**
     * Get the number of cards in the CardList
     * @return The number of Cards in the CardList
     */
    public int size() {
        return this.cards_.size();
    }

    /**
     * Checks wether the CardList is empty or not
     * @return True, if there are no Cards in the CardList
     */
    public boolean isEmpty() {
        return this.cards_.isEmpty();
    }

    /**
     * Checks wether the CardList contains the Card given
     * @return True, if the list contains the Card given
     * @param c The card
     */
    public boolean contains(Card c) {
        Iterator<Card> cardIter = this.iterator();
        while (cardIter.hasNext()) {
            Card next = cardIter.next();
            if (next.equals(c)) {
                return true;
            }
        }
        return false;

    }

    /**
     * Clear the CardList
     */
    public void clear() {
        this.cards_.clear();
    }

    /**
     * Convert the CardList to a java.util.list object
     * @return A List (java.util.List) of the Cards in CardList
     */
    public List<Card> list() {
        return this.cards_;
    }

    /**
     * Adds all Card from the CardList given to the CardList
     * @param list The CardList to add
     */
    public void addAll(CardList list) {
        this.cards_.addAll(list.list());
    }
    
    public String toString() {
        Iterator<Card> listIter = this.iterator();
        String result = "";
        while (listIter.hasNext()) {
            Card c = listIter.next();
            result += ESwissColorNames.getName(c.getColor()) + " " + ESwissValueNames.getName(c.getValue()) + "\n";

        }

        return result;

    }

	/**
	 * Dummy method: Used in GoneCardHeap only
	 */
	public CardList getGoneOfColor(int i) {
	
		return null;
	}
	
	
}