/*
 * Created on 16.03.2005
 *
 * Filename: GoneCardStack.java
 */
package jagafa.object;

import jagafa.stats.CardTool;

import java.util.LinkedList;
import java.util.List;

/**
 * GoneCardHeap is a CardList which holds all Cards that have been played during
 * a Game, including the winners of every turn
 * 
 */
public class GoneCardHeap extends CardList {
	private List<Board> boardList_;

	private List<Player> winnerList_;

	/**
	 * Constructor: Init the winner and board Lists
	 */
	public GoneCardHeap() {
		super();
		this.winnerList_ = new LinkedList<Player>();
		this.boardList_ = new LinkedList<Board>();
	}

	/**
	 * Adds the turn to the GoneCardHeap
	 * @param b The turns Board object
	 * @param winner The turns winner
	 */
	public void addTurn(Board b, Player winner) {
		this.winnerList_.add(winner);
		Board b2 = new Board(b);
		super.addAll(b2);
		this.boardList_.add(b2);
	}

	/**
	 * @return True, if the card is in the GoneCardHeap
	 */
	public boolean isGone(Card c) {
		return super.contains(c);
	}

	/**
	 * @return a CardList with all the Cards of the color given in the GoneCardHeap
	 * @param color The color number
	 */
	public CardList getGoneOfColor(int color) {
		CardList list = CardTool.getAllOfColor(this, color);
		return list;
	}

	/**
	 * @return The board played in Turn number i
	 * @param turnNr The turn number
	 */
	public Board getBoard(int turnNr) {
		return this.boardList_.get(turnNr);
	}

	/**
	 * @return The winner of the turn specified
	 * @param turnNr The turn number
	 */
	public Player getWinner(int turnNr) {
		return this.winnerList_.get(turnNr);
	}

	/**
	 * @return The number of turns stored in the GoneCardHeap
	 */
	public int turns() {
		return this.boardList_.size();
	}

	/**
	 * @return True, if the Player player has played "color" in the Board given
	 * @param player The relevant player
	 * @param board The relevant board
	 */
	public boolean hasPlayedColorIn(Player player, Board board) {
		int firstColor = board.getFirstColor();

		Card playerC = getPlayedCard(player, board);
		if (playerC.getColor() == firstColor) {
			return true;
		} else {
			return false;
		}

		
	}

	/**
	 * Get the card a specific Player played on a specific Board
	 * @return The Card Player p has played in the Board given
	 * @param player The relevant player
	 * @param b The relevant board
	 */
	public Card getPlayedCard(Player player, Board b) {
		int firstColor = b.get(0).getColor();

		for (int i = 0; i < b.size(); i++) {
			if (b.getPlayerOfPos(i).equals(player)) {

				Card playerC = b.get(i);
				return playerC;
			}
		}

		return null;
	}

}
