/*
 * Created on 24.02.2005 Filename: Hand.java
 */
package jagafa.object;

import jagafa.rule.RuleSet;

import java.util.Iterator;



/**
 * Hand is a CardList holding all the Cards in a Players hand
 */
public class Hand extends CardList {
   
	private static final int COLORS = 4;

	/**
	 * Constructor: Create a new Hand
	 */
    public Hand() {
        super();
       
    }
    /**
	 * Constructor: Init the Hand with the Cards in the CardList
	 */
	public Hand(CardList handCards) {
		
			this.addAll(handCards);

	}
	
	/**
	 * Sort the Hand according to the RuleSet
	 * @param rules The relevant RuleSet
	 */
	public void sort(RuleSet rules) {
		if (rules==null) {
			
			return;
		}
        CardList colorList[] = new CardList[COLORS];

        for (int i = 0; i < COLORS; i++) {
            colorList[i] = new CardList();
            Iterator<Card> cardIter = this.iterator();
            while (cardIter.hasNext()) {

                Card card = cardIter.next();
                if (card.getColor()==i) {
                    colorList[i].add(card);
                }
            }
        }
        this.clear();

        for (int i = 0; i < COLORS; i++) {
            this.addAll(sortByValue(colorList[i],rules));
        }

    }
    
	/**
	 * Sort a CardList by value according to a RuleSet
	 * @param rules The relevant RuleSet
	 * @param cardList The CardList to sort
	 */
    private CardList sortByValue(CardList cardList, RuleSet rules) {
        Card cardArray[] = new Card[cardList.size()];
        Board dummy = new Board();
        for (int i = 0;i<cardList.size();i++) {
            cardArray[i] = cardList.get(i);
        }
        
        for (int i = 0;i<cardList.size();i++) {
            for (int j = 0;j<cardList.size();j++) {
                int valueI = rules.getPower(cardArray[i],dummy);
                int valueJ = rules.getPower(cardArray[j],dummy);
                
                if (valueI>=valueJ && i<j) {
                    Card c = cardArray[i].copy();
                    cardArray[i] = cardArray[j].copy();
                    cardArray[j] = c.copy();
                } 
            }

        }
        
        CardList newList = new CardList();
        for (int i = 0;i<cardList.size();i++) {
            newList.add(cardArray[i].copy());
        }
        
                
        return newList;
    }
    
}