package jagafa.object;

import jagafa.JassRound;
import jagafa.ai.ProbabilityAI;
import jagafa.ai.core.JassAI;

/**
 * The Player class holds the hand and the AI of a player. It also knows wether
 * a he is a computer player or human. 
 * TODO: Needs refactoring: Add "partner" and "opponent" recognition to the Player class
 */
public class Player {

	/**
	 * Wenn der Spieler ein Mensch ist --> false
	 */
	private boolean isComputer_;

	/**
	 * Karten auf der Hand des Spielers
	 */
	private Hand hand_;

	/**
	 * Name des Spielers
	 */
	private String name_;

	/**
	 * Erreichte Punktzahl
	 */
	private int score_;

	private int numberOfCards_;

	private JassAI jassAI_;

	private static int players_ = 0;

	/**
	 * Erstellt neues Spielerobjekt
	 * @param isHuman True if this player is a human player
	 * @param numberOfCards The number of cards in the hand of the player
	 */
	public Player(int numberOfCards, boolean isHuman) {
		this.jassAI_ = new ProbabilityAI();
		isComputer_ = !isHuman;
		if (isHuman) {
			this.jassAI_ = null;
		}
		hand_ = new Hand();
		this.name_ = String.valueOf(players_);
		score_ = 0;
		this.numberOfCards_ = numberOfCards;

		players_++;
	}

	public Player(int numberOfCards, boolean isHuman, JassAI ai) {
		this(numberOfCards, isHuman);
		this.jassAI_ = ai;
		
	}

	/**
	 * Zhlt points zum aktuellen Punktestand dazu
	 * @param points The score to add
	 */
	public void addScore(int points) {
		score_ = score_ + points;
	}

	/**
	 * Get the players score
	 * @return Gibt Anzahl erreichter Punkte zurck
	 */
	public int getScore() {
		return score_;
	}

	/**
	 * Setzt Punktzahl auf Null zurck
	 */
	public void resetScore() {
		score_ = 0;

	}

	/**
	 * 
	 * @return Liefert einen String mit dem Namen des Spielers
	 */
	public String toString() {
		return this.name_;
	}

	/**
	 * Get the players hand object
	 * @return The Players hand object
	 */
	public Hand getHand() {
		return hand_;
	}

	/**
	 * Gibt die Handkarten auf dem Bildschirm aus
	 */
	public void printHand() {
		System.out.println(this.hand_);

	}

	/**
	 * Spielt die ite Karte aus der hand des Spielers auf das Board von game
	 * 
	 * @param i The card number in the players hand
	 * @param round The Game object to which the player plays the card
	 */
	public void playCard(int i, JassRound round) {

		Card toPlay = this.hand_.get(i);
		if (i >= this.hand_.size()) {
			return;
		}

		if (round.isValidPlay(toPlay)) {

			round.playCard(toPlay, this);
			this.hand_.remove(i);

		} else {
			System.out.println("Invalid Card!");
		}
	}

	/**
	 * Checks wether the player is AI-driven or not
	 * @return True, if the player is driven by AI
	 */
	public boolean isComputer() {
		return this.isComputer_;
	}

	/**
	 * Sets the name of the player
	 * @param newName The new name
	 */
	private void setName(String newName) {
		this.name_ = newName;
	}

	/**
	 * Get the player name
	 * @return A string with the name of the player
	 */
	public String getName() {
		return this.name_;
	}

	/**
	 * Get the players AI instance
	 * @return The AI of the Player
	 */
	public JassAI getAI() {
		return this.jassAI_;
	}

	/**
	 * Spielt eine Karte aus der hand des Spielers auf das Board von game
	 * 
	 * @param card The card
	 * @param round The Game object to which the player plays the card
	 */
	public boolean playCard(Card card, JassRound round) {
		if (card == null) {
			return false;
		}

		if (round.isValidPlay(card)) {
			round.playCard(card, this);
			this.hand_.remove(card);

		} else {
			System.out.println("Invalid Card!");
			return false;

		}
		return true;
	}

	public boolean equals(Object sec) {
		if (!(sec instanceof Player)) {
			return false;
		}
		Player other = (Player) sec;
		if (other.getName().equals(this.getName())) {
			return true;
		}
		return false;

	}

	public int hashCode() {
		int sum = 0;
		char carray[] = this.getName().toCharArray();
		for (int i = 0; i < carray.length; i++) {
			sum += (int) carray[i];
		}

		return sum;
	}

	public void setAI(JassAI simpleAI) {
		this.jassAI_ = simpleAI;

	}

	public static void resetPlayerCount() {
		players_ = 0;
	}

	public int getID() {
		return Integer.valueOf(this.getName()).intValue();
	}
}