/**
 * 
 */
package jagafa.rule;

import jagafa.naming.EValueNames;
import jagafa.object.Board;
import jagafa.object.Card;
import jagafa.object.Hand;
import jagafa.stats.CardTool;

import java.util.Iterator;

/**
 * @author Besitzer
 * 
 */
public class CoiffeurRules implements RuleSet {
	private enum ObenHierarchy {
		V10, VJ, V9, VQ, V8, VK, V7, VAS, V6;
		public static String getName(int i) {
			return values()[i].name().substring(1);
		}
	}

	public Card getHigherCard(Card c1, Card c2) {
		Board dummy = new Board();
		if (this.getPower(c1, dummy) > this.getPower(c2, dummy)) {
			return c1;
		} else {
			return c2;
		}
	}

	public int getPower(Card c, Board b) {
		int power = 0;

		power = this.getValueRank(c, b);

		power += 100 * this.getColorRank(c, b);

		return power;
	}

	public int getScore(Card c) {
		switch (ObenHierarchy.values()[c.getValue()]) {
		case V6:
			return 0;
		case V7:
			return 0;
		case V8:
			return 8;
		case V9:
			return 0;
		case V10:
			return 10;
		case VJ:
			return 2;
		case VQ:
			return 3;
		case VK:
			return 4;
		case VAS:
			return 11;
		}

		return 0;
	}

	public boolean isValid(Card cardToCheck, Hand hand, Board board) {
		if (board.cardsPlayed() > 0) {
			Card firstCard = board.get(0);
			int firstColor = firstCard.getColor();

			// Wenn keine Karten von der startFarbe auf der hand, dann egal
			// welche Karte gespielt wird
			if (CardTool.getAllOfColor(hand, firstColor).size() == 0) {
				return true;
			} else {
				// Wenn noch Karten von der startFarbe, dann muss Karte von der
				// gleichen Farbe sein
				if (cardToCheck.getColor() == (firstColor)) {
					return true;
				} else {
					return false;
				}
			}

		}

		return true;
	}

	public int getValueRank(Card c, Board b) {
		for (int i = 0; i < 9; i++) {
			if (ObenHierarchy.getName(i).equals(
					EValueNames.getName(c.getValue()))) {
				return 8 - i;
			}
		}
		return -1;
	}

	public int getColorRank(Card c, Board b) {
		if (b.cardsPlayed() > 0) {
			Card firstCard = b.get(0);
			int firstColor = firstCard.getColor();
			if (firstColor == c.getColor()) {
				return 2;
			} else {
				return 1;
			}
		}

		return 2;
	}

	public boolean compareHigher(Card c1, Board b) {
		int posc1 = getPower(c1, b);
		int highestOnBoard = 0;

		Iterator<Card> boardIter = b.iterator();

		while (boardIter.hasNext()) {
			Card nextC = boardIter.next();
			int posBoard = this.getPower(nextC, b);
			if (posBoard > highestOnBoard) {
				highestOnBoard = posBoard;
			}
		}

		if (posc1 >= highestOnBoard) {
			return true;
		}
		return false;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jass.rule.Rules#getName()
	 */
	public String getName() {
		// TODO Auto-generated method stub
		return "Coiffeur";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jass.rule.Rules#specialColor()
	 */
	public int specialColor() {
	
		return -1;
	}
	
	public int hashCode()
	{
		int sum = 0;
		char carray[] = this.getName().toCharArray();
		for (int i = 0;i<carray.length;i++) {
			sum += (int) carray[i];
		}
		
		return sum;
	}

	public void updateAfterTurn() {
		// TODO Auto-generated method stub
		
	}
}
