/**
 * 
 */
package jagafa.rule;

import jagafa.naming.ESwissColorNames;
import jagafa.naming.EValueNames;
import jagafa.object.Board;
import jagafa.object.Card;
import jagafa.object.CardList;
import jagafa.object.Hand;
import jagafa.stats.CardTool;

import java.util.Iterator;

/**
 * @author Besitzer
 * 
 */
public class KijRules implements RuleSet {
	private int trumpfColor_;
	
	private enum ObenHierarchy {
		CAS, CK, CQ, CJ, C10, C9, C8, C7, C6;
	}
	
	private enum TrumpfHierarchy {
		V6,V7,V8,V10,VQ,VK,VAS,V9,VJ;

		
		
		/**
		 * @return
		 */
		public static String getName(int i) {
			
				return values()[i].name().substring(1);
			}
		
	
	}

	public KijRules(int trumpfColor){
		this.trumpfColor_ = trumpfColor;
	}
	
	public Card getHigherCard(Card c1, Card c2) {
		Board dummy = new Board();
		if (this.getPower(c1, dummy) > this.getPower(c2, dummy)) {
			return c1;
		} else {
			return c2;
		}
	}

	public int getPower(Card c, Board b) {
		int power = 0;

		power = this.getValueRank(c, b);

		power += 15 * this.getColorRank(c, b);
		
		/*if (c.getColor()==trumpfColor_){
			int value = c.getValue();
			if (value == ObenHierarchy.CJ.ordinal()){
				power *= 500;
			}
			if (value == ObenHierarchy.C9.ordinal()){
				power *= 250;
			}
		
		}*/
		return power;
	}

	public int getScore(Card c) {
		int color = c.getColor();
		switch (ObenHierarchy.values()[c.getValue()]) {
		case C6:
			return 0;
		case C7:
			return 0;
		case C8:
			return 0;
		case C9:
			if (color==trumpfColor_){
				return 14;
			}
			return 0;
		case C10:
			return 10;
		case CJ:
			if (color==trumpfColor_){
				return 20;
			}
			
			return 2;
		case CQ:
			return 3;
		case CK:
			return 4;
		case CAS:
			return 11;
		}

		return 0;
	}

	public boolean isValid(Card cardToCheck, Hand hand, Board board) {
		if (cardToCheck.getColor()==trumpfColor_){
			return true;
		}
		if (board.cardsPlayed() > 0) {
			Card firstCard = board.get(0);
			int firstColor = firstCard.getColor();

			// Wenn keine Karten von der startFarbe auf der hand, dann egal
			// welche Karte gespielt wird
			if (firstColor==trumpfColor_){
				CardList caList = CardTool.getAllOfColor(hand, firstColor);
				if (caList.size()==1 && caList.contains(new Card(trumpfColor_,EValueNames.VJ.ordinal()))){
					return true;
				}
			}
			
			if (CardTool.getAllOfColor(hand, firstColor).size() == 0) {
				return true;
			} else {
				// Wenn noch Karten von der startFarbe, dann muss Karte von der
				// gleichen Farbe sein
				if (cardToCheck.getColor() == (firstColor)) {
					return true;
				} else {
					
					return false;
				}
			}

		}

		return true;
	}

	
	public int getValueRank(Card c, Board b) {
		if (c.getColor()==trumpfColor_){
			for (int i = 0;i<9;i++){
				if (TrumpfHierarchy.getName(i).equals(EValueNames.getName(c.getValue()))) {
					return TrumpfHierarchy.values()[i].ordinal();
				}
			}
		}
		
		for (ObenHierarchy o : ObenHierarchy.values()) {
			if (o.ordinal() == c.getValue()) {
				return 8-o.ordinal();
			}
		}
		return -1;
	}

	
	public int getColorRank(Card c, Board b) {
		if (c.getColor()==trumpfColor_){
			
			return 5;
		}
		
		if (b.cardsPlayed() > 0) {
			Card firstCard = b.get(0);
			int firstColor = firstCard.getColor();
			
			if (firstColor == c.getColor()) {
				
				return 2;
			} else {
				return 1;
			}
		}

		return 2;
	}

	
	public boolean compareHigher(Card c1, Board board) {
        int posc1 = getPower(c1, board);
        int highestOnBoard = 0;

        Iterator<Card> boardIter = board.iterator();

        while (boardIter.hasNext()) {
            Card nextC = boardIter.next();
            int posBoard = this.getPower(nextC, board);
            if (posBoard > highestOnBoard) {
                highestOnBoard = posBoard;
            }
        }

        if (posc1 >= highestOnBoard) {
            return true;
        }
        return false;

    }

	/* (non-Javadoc)
	 * @see jass.rule.Rules#getName()
	 */
	public String getName() {
		return ESwissColorNames.getName(this.trumpfColor_) + " Kij";
		
	}
	/* (non-Javadoc)
	 * @see jass.rule.Rules#specialColor()
	 */
	public int specialColor() {
		// TODO Auto-generated method stub
		return this.trumpfColor_;
	}
	
	public int hashCode()
	{
		int sum = 0;
		char carray[] = this.getName().toCharArray();
		for (int i = 0;i<carray.length;i++) {
			sum += (int) carray[i];
		}
		
		return sum;
	}

	public void updateAfterTurn() {
		// TODO Auto-generated method stub
		
	}

}
