/*
 * Created on 25.02.2005 Filename: Rules.java
 */
package jagafa.rule;

import jagafa.object.Board;
import jagafa.object.Card;
import jagafa.object.Hand;

/**
 * Interface for defining a RuleSet (eg. Oben, Unten, Trumpf)
 * All Methods have to be implemented
 */
public interface RuleSet {

	/* TODO: Add interface method for in-round rule change */
	
    /** 
     * Gibt die bessere der beiden Karten zurck
     *  @return The card with the higher power
     *  @param c1 The first card
     *  @param c2 The second card    
     */
    public abstract Card getHigherCard(Card c1, Card c2);

    /**
     * Check wether a card beats all the cards on a board given
     * @param c1 The relevant card
     * @param b The relevant board
     * @return True, if the card specified beats all the cards in the Board
     */
    public boolean compareHigher(Card c1, Board b);
   
    
    /** 
     * Liefert den Wert der Karte relativ zum liegenden Board
     * @return The power of a card with respect to a Board
     * @param c The relevant card
     * @param b The relevant Board     
     */
    public abstract int getPower(Card c,  Board b);

    /** 
     * Liefert den Punktewert der Karte
     * @return The score of a card
     * @param c The relevant card 
     */
    public abstract int getScore(Card c);

    /** 
     * Prft die Gltigkeit der Karte bei gegebenem Board und Hand 
     * @return True, if the card is a valid play
     * @param c The card to check
     * @param h The hand holding the card
     * @param b The Board on which the cards wants to played
     */
    public abstract boolean isValid(Card c, Hand h, Board b);
    
    /** 
     * Liefert den internen Wert des Wertes der Karte bei gegebenem Board
     * @return An internal Rank of the cards value compared to other cards on a board
     * @param b The relevant board
     * @param c The relevant card 
     */
    public abstract int getValueRank(Card c, Board b);
    
    /** 
     * Liefert den internen Wert der Farbe der Karte bei gegebenem Board
     * @return An internal Rank of the cards color compared to other cards on a board
     * @param b The relevant board
     * @param c The relevant card 
     */
    public abstract int getColorRank(Card c, Board b);
    
    /** 
     * Liefert den Namen des RuleSet 
     * @return A String containing the name of the RuleSet 
     */
    public abstract String getName();
    
    /** 
     * Liefert die Spezialfarbe falls bentigt 
     * @return The color with a special role in the RuleSet
     */
    public abstract int specialColor();
    
    /**
     * Update the RuleSet after each turn (for automatic Changes)
     */
    public abstract void updateAfterTurn();
}