/**
 * 
 */
package jagafa.rule;

import jagafa.object.Board;
import jagafa.object.Card;
import jagafa.object.Hand;
import jagafa.stats.CardTool;

/**
 * @author Besitzer
 * 
 */
public class SlalomObenStartRules implements RuleSet {
	private enum UntenHierarchy {
		C6, C7, C8, C9, C10, CJ, CQ, CK, CAs;
	}

	private enum ObenHierarchy {
		CAs, CK, CQ, CJ, C10, C9, C8, C7, C6;
	}

	private static final int UNTEN = 0;

	private static final int OBEN = 1;

	private int direction_;

	private int startDirection_;

	private ObenRules oben_;
	private UntenRules unten_;
	
	public Card getHigherCard(Card c1, Card c2) {
		if (this.direction_==OBEN) {
			return this.oben_.getHigherCard(c1,c2);
		} else {
			return this.unten_.getHigherCard(c1,c2);
		}
	}

	public SlalomObenStartRules() {
		this.startDirection_ = OBEN;
		this.direction_ = OBEN;
		this.oben_ = new ObenRules();
		this.unten_ = new UntenRules();
	}

	public int getPower(Card c, Board b) {
		
		int power = 0;

		power = this.getValueRank(c, b);

		power += 100 * this.getColorRank(c, b);

		return power;
	}

	public int getScore(Card c) {
		if (this.startDirection_==OBEN) {
			return this.oben_.getScore(c);
		} else {
			return this.unten_.getScore(c);
		}
	}

	public boolean isValid(Card cardToCheck, Hand hand, Board board) {
		if (cardToCheck == null) {
			return false;
		}
		if (board.cardsPlayed() > 0) {
			Card firstCard = board.get(0);
			int firstColor = firstCard.getColor();

			// Wenn keine Karten von der startFarbe auf der hand, dann egal
			// welche Karte gespielt wird
			if (CardTool.getAllOfColor(hand, firstColor).size() == 0) {
				return true;
			} else {
				// Wenn noch Karten von der startFarbe, dann muss Karte von der
				// gleichen Farbe sein
				if (cardToCheck.getColor() == (firstColor)) {
					return true;
				} else {
					return false;
				}
			}

		}

		return true;
	}

	public int getValueRank(Card c, Board b) {
		if (this.direction_==OBEN) {
			return this.oben_.getValueRank(c,b);
		} else {
			return this.unten_.getValueRank(c,b);
		}
	}

	public int getColorRank(Card c, Board b) {
		if (this.direction_==OBEN) {
			return this.oben_.getColorRank(c,b);
		} else {
			return this.unten_.getColorRank(c,b);
		}
	}

	public boolean compareHigher(Card c1, Board b) {
		if (this.direction_==OBEN) {
			return this.oben_.compareHigher(c1,b);
		} else {
			return this.unten_.compareHigher(c1,b);
		}
		

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jass.rule.Rules#getName()
	 */
	public String getName() {
		if (this.direction_==OBEN) {
			return "Slalom (Oben)";	
		} else {
			return "Slalom (Unten)";
		}
		
	}

	/* (non-Javadoc)
	 * @see jass.rule.Rules#specialColor()
	 */
	public int specialColor() {
		// TODO Auto-generated method stub
		return -1;
	}

	public int hashCode() {
		int sum = 0;
		char carray[] = this.getName().toCharArray();
		for (int i = 0; i < carray.length; i++) {
			sum += (int) carray[i];
		}

		return sum;
	}
	
	
	public void updateAfterTurn() {
		if (this.direction_==OBEN) {
			this.direction_ = UNTEN;
		} else {
			this.direction_ = OBEN;
		}
	}
}
