/**
 * 
 */
package jagafa.rule;

import jagafa.object.Board;
import jagafa.object.Card;
import jagafa.object.Hand;
import jagafa.stats.CardTool;

import java.util.Iterator;

/**
 * @author Besitzer
 * 
 */
public class UntenRules implements RuleSet {
	private enum UntenHierarchy {
		C6, C7, C8, C9, C10, CJ, CQ, CK, CAs;
	}

	public Card getHigherCard(Card c1, Card c2) {
		Board dummy = new Board();
		if (this.getPower(c1, dummy) > this.getPower(c2, dummy)) {
			return c1;
		} else {
			return c2;
		}
	}

	public int getPower(Card c, Board b) {
		int power = 0;

		power = this.getValueRank(c, b);

		power += 100 * this.getColorRank(c, b);

		return power;
	}

	public int getScore(Card c) {
		switch (UntenHierarchy.values()[c.getValue()]) {
		case C6:
			return 11;
		case C7:
			return 0;
		case C8:
			return 8;
		case C9:
			return 0;
		case C10:
			return 10;
		case CJ:
			return 2;
		case CQ:
			return 3;
		case CK:
			return 4;
		case CAs:
			return 0;
		}

		return 0;
	}

	public boolean isValid(Card cardToCheck, Hand hand, Board board) {
		if (cardToCheck==null){
			return false;
		}
		if (board.cardsPlayed() > 0) {
			Card firstCard = board.get(0);
			int firstColor = firstCard.getColor();

			// Wenn keine Karten von der startFarbe auf der hand, dann egal
			// welche Karte gespielt wird
			if (CardTool.getAllOfColor(hand, firstColor).size() == 0) {
				return true;
			} else {
				// Wenn noch Karten von der startFarbe, dann muss Karte von der
				// gleichen Farbe sein
				if (cardToCheck.getColor() == (firstColor)) {
					return true;
				} else {
					return false;
				}
			}

		}

		return true;
	}

	public int getValueRank(Card c, Board b) {
		for (UntenHierarchy o : UntenHierarchy.values()) {
			if (o.ordinal() == c.getValue()) {
				return 8-o.ordinal();
			}
		}
		return -1;
	}

	public int getColorRank(Card c, Board b) {
		if (b.cardsPlayed() > 0) {
			Card firstCard = b.get(0);
			int firstColor = firstCard.getColor();
			if (firstColor == c.getColor()) {
				return 2;
			} else {
				return 1;
			}
		}

		return 2;
	}

	public boolean compareHigher(Card c1, Board b) {
		int posc1 = getPower(c1, b);
		int highestOnBoard = 0;

		Iterator<Card> boardIter = b.iterator();

		while (boardIter.hasNext()) {
			Card nextC = boardIter.next();
			int posBoard = this.getPower(nextC, b);
			if (posBoard > highestOnBoard) {
				highestOnBoard = posBoard;
			}
		}

		if (posc1 >= highestOnBoard) {
			return true;
		}
		return false;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jass.rule.Rules#getName()
	 */
	public String getName() {
		// TODO Auto-generated method stub
		return "Unten";
	}
	
	/* (non-Javadoc)
	 * @see jass.rule.Rules#specialColor()
	 */
	public int specialColor() {
		// TODO Auto-generated method stub
		return -1;
	}
	
	public int hashCode()
	{
		int sum = 0;
		char carray[] = this.getName().toCharArray();
		for (int i = 0;i<carray.length;i++) {
			sum += (int) carray[i];
		}
		
		return sum;
	}

	public void updateAfterTurn() {
		// TODO Auto-generated method stub
		
	}
}
