package jagafa.scores;

import jagafa.rule.RuleSet;

import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public abstract class AbstractScoreTable implements ScoreTable {
	private static final int PLAYERS = 4;

	private List<Object> roundScores_;

	private List<RuleSet> roundRules_;

	private Map<String, RuleSet> availableRules_;

	private int scores_[];

	private int startPlayer_;

	private List<Integer> roundChooser_;

	protected int choosingPlayer_;

	private boolean canChangePlayer_ = true;

	public AbstractScoreTable() {
		scores_ = new int[PLAYERS];
		this.roundRules_ = new LinkedList<RuleSet>();
		this.availableRules_ = new LinkedHashMap<String, RuleSet>();
		this.roundScores_ = new LinkedList<Object>();
		this.roundChooser_ = new LinkedList<Integer>();
		this.startPlayer_ = 0;
	}

	public boolean isRuleAvailable(RuleSet rule) {

		return (this.availableRules_.get(rule.getName()) != null);
	}

	public void activateRule(RuleSet rule) {
		this.availableRules_.put(rule.getName(), rule);
	}

	public void deactivateRule(RuleSet rule) {
		this.availableRules_.put(rule.getName(), null);

	}

	public List<RuleSet> getRules() {
		List<RuleSet> l = new LinkedList<RuleSet>();
		RuleSet arr[] = new RuleSet[this.availableRules_.size()];

		this.availableRules_.values().toArray(arr);

		for (int i = 0; i < arr.length; i++) {
			l.add(arr[i]);
		}

		return l;
	}

	public void addScores(int[] scores, RuleSet rules) {
		if (scores.length != scores_.length) {
			System.err.println("Error: Not a valid score array");
		}
		this.roundScores_.add(scores);
		this.roundRules_.add(rules);
		this.roundChooser_.add(this.getStartPlayer());
		for (int i = 0; i < scores_.length; i++) {
			this.scores_[i] += scores[i];
		}
	}

	public int[] getScores() {
		return this.scores_;
	}

	public int getStartPlayer() {
		return this.startPlayer_;
	}

	public void setStartPlayer(int pindex) {
		if (pindex >= 4) {
			pindex -= 4;
		}
		this.startPlayer_ = pindex;
		this.choosingPlayer_ = this.startPlayer_;
	}

	public void resetScores() {
		scores_ = new int[PLAYERS];
		this.roundRules_ = new LinkedList<RuleSet>();
		this.roundScores_ = new LinkedList<Object>();
		this.roundChooser_ = new LinkedList<Integer>();
		this.startPlayer_ = 0;
		this.setChoosingPlayer(this.startPlayer_);
	
	}

	protected void setChoosingPlayer(int i) {
		this.choosingPlayer_ = i;
		
	}

	public int roundsPlayed() {

		return this.roundScores_.size();
	}

	public String toString() {
		String res = new String();
		float mean1 = 0;
		float mean2 = 0;
		for (int i = 0; i < this.roundRules_.size(); i++) {
			RuleSet roundRule = this.roundRules_.get(i);
			int scores[] = (int[]) this.roundScores_.get(i);

			int team1 = scores[0] + scores[2];
			int team2 = scores[1] + scores[3];

			res += "Round   " + i + "\tTeam1: " + team1;
			res += " Team2: " + (team2);
			res += "\tChoice: " + this.roundChooser_.get(i) + "\t" + roundRule.getName();
			res += "\n";

			int chooser = this.roundChooser_.get(i);
			if (chooser == 0 || chooser == 2) {
				mean1 += team1;
			} else {
				mean2 += team2;
			}
		}

		mean1 /= this.roundsPlayed() / 2;
		mean2 /= this.roundsPlayed() / 2;

		int team1 = this.getScores()[0] + this.getScores()[2];
		int team2 = this.getScores()[1] + this.getScores()[3];

		res += "---------------------------------------------------------------------------------------------\n";
		res += "Sum:      " + "\tTeam1: " + (team1);
		res += " Team2: " + (team2) + "\n";

		res += "Mean:     " + "\tTeam1: " + (mean1);
		res += " Team2: " + (mean2) + "\n";

		return res;
	}

	/**
	 * Does nothing by default
	 */
	public boolean changeChoosingPlayer() {
		return false;
	}

	public boolean canChangeChoosingPlayer() {
		
		return this.canChangePlayer_;
	}
	
	protected void setCanChangePlayer(boolean canChange) {
		this.canChangePlayer_ = canChange;
	}
	
	
}
