package jagafa.scores;

import jagafa.rule.CoiffeurRules;
import jagafa.rule.ObenRules;
import jagafa.rule.RuleSet;
import jagafa.rule.UntenRules;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

public class SchieberScoreTable extends AbstractScoreTable implements ScoreTable {

	private static final Integer SINGLE = new Integer(1);

	private static final Integer DOUBLE = new Integer(2);

	private int winningScore_;

	private Map<RuleSet, Integer> rlist;

	public SchieberScoreTable(int winningScore) {
		super();
		this.setCanChangePlayer(true);
		this.winningScore_ = winningScore;
		rlist = new LinkedHashMap<RuleSet, Integer>();
		rlist.put(new ObenRules(), SINGLE);
		rlist.put(new UntenRules(), SINGLE);
		/*rlist.put(new TrumpfRules(0), SINGLE);
		rlist.put(new TrumpfRules(1), SINGLE);
		rlist.put(new TrumpfRules(2), SINGLE);
		rlist.put(new TrumpfRules(3), SINGLE);*/
		//rlist.put(new SlalomObenStartRules(),SINGLE);
		rlist.put(new CoiffeurRules(), SINGLE);
		Iterator<RuleSet> ruleIter = rlist.keySet().iterator();

		while (ruleIter.hasNext()) {
			this.activateRule(ruleIter.next());
		}
		this.choosingPlayer_ = this.getStartPlayer();
	}

	@Override
	public void addScores(int[] scores, RuleSet rules) {
		if (rules==null) {
			return;
		}
		/* TODO: Add functionality for MATCH score */
		int mult = this.getMultiplier(rules);

		for (int i = 0; i < scores.length; i++) {
			scores[i] *= mult;
		}
		super.addScores(scores, rules);
		this.setCanChangePlayer(true);
		int roundsPlayed = this.roundsPlayed();
		
		while (roundsPlayed>=4) {
			roundsPlayed-=4;
		}
		this.setStartPlayer(roundsPlayed);
		
	}

	public boolean isGameOver() {
		int scores[] = this.getScores();

		int scoreTeam1 = scores[0] + scores[2];
		int scoreTeam2 = scores[1] + scores[3];

		if (scoreTeam1 >= this.winningScore_) {
			this.setCanChangePlayer(true);
			return true;
		}
		if (scoreTeam2 >= this.winningScore_) {
			this.setCanChangePlayer(true);
			return true;
		}
		return false;
	}

	public int getMultiplier(RuleSet rule) {
		return (Integer) this.rlist.get(rule);
	}

	
	/**
	 * Schieben, falls noch mglich
	 */
	public boolean changeChoosingPlayer() {
		
		if (this.canChangeChoosingPlayer()) {
			int starter = this.getStartPlayer();
			
			
			starter += 2;
			while (starter>=4) {
				starter -= 4;
			}
			
			this.setChoosingPlayer(starter);
			this.setStartPlayer(starter);
			this.setCanChangePlayer(false);
			System.out.println("Hand over!");
			return true;
		} else {
			return false;
		}

	}

	
	
	
	public int getChoosingPlayer() {
		if (this.choosingPlayer_==-1) {
			this.choosingPlayer_ = this.getStartPlayer();
		}
		
		return this.choosingPlayer_;
	}

	
	public int getStartPlayer() {
		if (!this.canChangeChoosingPlayer()) {
			return this.getChoosingPlayer();
		}
		
		return super.getStartPlayer();
	}
	
	public int getWinningScore() {
		return this.winningScore_;
	}

}
