package jagafa.scores;

import jagafa.rule.RuleSet;

import java.util.List;

public interface ScoreTable {
	public abstract boolean isRuleAvailable(RuleSet rule);
	
	/**
	 * Activate a RuleSet. The RuleSet can be chosen after activation
	 * @param rule
	 */
	public abstract void activateRule(RuleSet rule);
	
	/**
	 * Deactivate a RuleSet. The RuleSet is not a valid choice afterwards.
	 * @param rule
	 */
	public abstract void deactivateRule(RuleSet rule);
	
	/**
	 * Get a list of all activated RuleSets
	 * @return
	 */
	public abstract List<RuleSet> getRules();
	
	
	/**
	 * Add scores achieved with a specific RuleSet to the table.
	 * @param scores
	 * @param rules
	 */
	public abstract void addScores(int scores[], RuleSet rules);
	
	/**
	 * Get all the players scores from all the rounds played
	 * @return
	 */
	public abstract int[] getScores();
	
	/**
	 * Reset all scores to 0
	 *
	 */
	public abstract void resetScores();
	
	/**
	 * Check wether the Game is over or not
	 * @return
	 */
	public abstract boolean isGameOver();

	
	/**
	 * Set the player which can start the next round
	 * @param p
	 */
	public abstract void setStartPlayer(int pindex);
	
	/**
	 * Get the player which can start the next round
	 */
	public abstract int getStartPlayer();
	
	/**
	 * Change internally the player which can choose the RuleSet.
	 * (Often: Triggered by a Player who doesn't want to choose -> hand the 
	 * choice over to another player) 
	 */
	public abstract boolean changeChoosingPlayer();
	
	public abstract boolean canChangeChoosingPlayer();
	/**
	 * Get the player which can choose the ruleSet for the next round
	 * @return The next choosing Player
	 */
	public abstract int getChoosingPlayer();
	
	/**
	 * Returns the multiplier of an activated RuleSet.
	 * The scores added by addScores() will be multiplied by this factor for the 
	 * RuleSet. (initial multiplier is 1)
	 * @param rule The RuleSet of which the scores will be multiplied
	 * @return The Multiplier (int)
	 */
	public abstract int getMultiplier(RuleSet rule);

	public abstract int roundsPlayed();

	public abstract int getWinningScore();
	
	
}
