/*
 * Created on 16.03.2005 Filename: AIMethods.java
 */
package jagafa.stats;

import jagafa.object.Board;
import jagafa.object.Card;
import jagafa.object.CardList;
import jagafa.object.Hand;
import jagafa.rule.RuleSet;

import java.util.Iterator;

/**
 * CardTool contains some helper Methods to deal with cards and CardLists
 */
public final class CardTool {
	
	/**
	 * Get all Cards in the CardList with the color given
	 * @return A CardList containing all Cards with the color specified from another CardList
	 * @param cards The source CardsList
	 * @param color The relevant color
	 */
	public static CardList getAllOfColor(CardList cards, int color) {
		boolean debug = true;

		CardList colorList = new CardList();
		Iterator<Card> cardIter = cards.iterator();

		while (cardIter.hasNext()) {
			Card nextC = cardIter.next();

			if (nextC.getColor() == (color)) {
				colorList.add(nextC);
			}

		}
		return colorList;
	}

	/** 
	 * Check wether the card is a Bock of level n or not
	 * @return True, if the Card can only be beaten by n(th) Cards left in the game
	 * @param nth The level of the Bock
	 * @param c The card to check
	 * @param gone A CardList of all cards gone in the game
	 * @return rules The RuleSet to apply
	 */
	public static boolean isBock(int nth, Card c, CardList gone, RuleSet rules) {
		Iterator<Card> goneIter = gone.iterator();
		int numberOfHigherGone = 0;
		Board board = new Board();
		board.addCard(c, null);

		while (goneIter.hasNext()) {
			Card nextC = goneIter.next();
			if (nextC != null) {
				if (rules.compareHigher(nextC, board)
						&& nextC.getColor() == c.getColor()) {
					numberOfHigherGone++;
				}
			}
		}

		if (numberOfHigherGone == (8 - nth) - rules.getValueRank(c, board)) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Get all Valid Cards out of the Hand for the board with 
	 * the Ruleset specified
	 * @return A CardList containing all card out of A hand which can be played onto the board
	 * @param hand The relevant hand
	 * @param board The relevant board
	 * @param rules The RuleSet to apply
	 */
	public static CardList getValidCards(Hand hand, Board board, RuleSet rules) {
		CardList cardL = new CardList();

		for (int i = 0; i < hand.size(); i++) {
			Card c = hand.get(i);
			if (rules.isValid(c, hand, board)) {
				cardL.add(c);
			}
		}

		return cardL;
	}

}