package jagafa.stats;

import jagafa.JassRound;
import jagafa.object.Card;
import jagafa.object.CardList;
import jagafa.object.GoneCardHeap;
import jagafa.object.Player;
import jagafa.rule.RuleSet;

import javax.swing.JComponent;

public class PlayerStats {

	private static final int COLORS = 4;

	private CardList bockList_[];

	private CardList trumpfList_;

	private CardList colorList_[];

	private CardList validList_;

	private JassRound round_;

	private Player player_;

	private GoneCardHeap goneHeap_;

	public PlayerStats(JassRound round, Player player) {
		this.round_ = round;
		this.player_ = player;
		this.goneHeap_ = round.getGoneHeap();
		this.update();
	}
	
	public PlayerStats(RuleSet rules, Player player) {
		this.round_ = new JassRound(rules);
		this.goneHeap_ = round_.getGoneHeap();
		this.player_ = player;
		this.update();
	}

	/**
	 * Update the stats
	 */
	public synchronized void update() {
		if (this.round_ != null) {
			this.createColorList();
			if (this.round_.getRules()==null) {
				return;
			}
			
			
			this.createBockList();
			this.createValidList();
			this.createTrumpfList();
		}
	}

	private void createTrumpfList() {
		CardList cards = player_.getHand();
		this.trumpfList_ = new CardList();

		for (int i = 0; i < cards.size(); i++) {
			Card c = cards.get(i);
			if (c.getColor() == this.round_.getRules().specialColor()) {
				this.trumpfList_.add(c);
			}
		}

	}

	private void createValidList() {
		CardList cards = player_.getHand();
		this.validList_ = new CardList();

		for (int i = 0; i < cards.size(); i++) {
			Card c = cards.get(i);
			if (this.round_.isValidPlay(c)) {
				this.validList_.add(c);
			}
		}

	}

	/**
	 * Create lists with each containing the cards of a color of the active Players hand
	 */
	private void createColorList() {
		CardList cards = player_.getHand();
		this.colorList_ = new CardList[COLORS];

		// All Cards of Color i
		for (int i = 0; i < COLORS; i++) {
			this.colorList_[i] = new CardList();
			CardList colorC = CardTool.getAllOfColor(cards, i);
			this.colorList_[i].addAll(colorC);
		}
	}

	/**
	 * Create lists with Bocks sorted by its level and color
	 */
	private void createBockList() {
		if (round_.getRules().specialColor() != -1) {
			String trumpfe = "Anzahl Trmpfe: ";
			CardList cardsL = player_.getHand();
			this.trumpfList_ = CardTool.getAllOfColor(cardsL, round_.getRules()
					.specialColor());
		}

		bockList_ = new CardList[9];
		CardList cardsL = player_.getHand();
		for (int j = 0; j < 9; j++) {
			bockList_[j] = new CardList();
			for (int i = 0; i < cardsL.size(); i++) {
				if (CardTool.isBock(j, cardsL.get(i), this.goneHeap_, round_
						.getRules())) {
					bockList_[j].add(cardsL.get(i));
				}
			}
		}
	}

	/**
	 * @return Returns the bockList array where the array has the form arr[level]
	 */
	public CardList[] getBockList() {
		return this.bockList_;
	}

	/**
	 * @return Returns the colorList in the form CardList[color].
	 */
	public CardList[] getColorList() {
		return this.colorList_;
	}

	/**
	 * @return Returns the trumpfList
	 */
	public CardList getTrumpfList() {
		return this.trumpfList_;
	}

	public int countColorCards(int color) {
		return this.colorList_[color].size();
	}

	/**
	 * @return
	 */
	public CardList getValidList() {
		return this.validList_;
	}

	public void setRound(JassRound round) {
		this.round_ = round;
	}

	public JComponent getView() {
		return null;
	}
	
	public Player getPlayer() {
		return this.player_;
	}
	
	public void setPlayer(Player p) {
		this.player_ = p;
	}

}
