package jagafa.util.view;

import java.awt.Component;
import java.awt.GridLayout;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JPanel;

public class GridPanel extends JPanel {
	private List<RowPanel> rows_;

	/**
	 * 
	 */
	private static final long serialVersionUID = -56984613549427868L;

	public GridPanel() {
		super();
		this.rows_ = new LinkedList<RowPanel>();
	}

	/** 
	 * Update the GridPanel. Called after every insert or delete.
	 *
	 */
	private void update() {
		if (this.rowCount() == 0) {
			return;
		}
		this.removeAll();
		
		this.setLayout(new GridLayout(this.rowCount(), 1));
	
		for (int i = 0; i < this.rowCount(); i++) {
			RowPanel row = this.getRowInternal(i);
			this.add(row);
		}
	}
	
	/**
	 * Returns the component at the n-th row
	 */
	public JPanel getComponent(int row) {
		if (row >= this.rows_.size()) {
			return null;
		}
		return this.rows_.get(row);
	}

	/**
	 * Add a new row with n columns
	 * @param columns
	 */
	public void addRow(int columns) {
		RowPanel row = new RowPanel(columns);
		this.rows_.add(row);
		update();
	}

	/**
	 * Add a row with a single column
	 *
	 */
	public void addRow() {
		this.addRow(1);

	}
	
	/**
	 * Add the RowPanel specified
	 * @param row
	 */
	public void addRow(RowPanel row) {
		this.rows_.add(row);

	}
	

	/**
	 * Remove the row at index i
	 * @param index
	 */
	public void removeRow(int index) {
		if (index >= this.rows_.size()) {
			return;
		}
		this.rows_.remove(index);
		update();
	}

	/**
	 * Get the number of rows in the panel
	 */
	public int rowCount() {
		return this.rows_.size();
	}

	/**
	 * Set the component at (column, row) to the component specified
	 * @param column
	 * @param row
	 * @param component
	 */
	public void setComponent(int column, int row, JComponent component) {
		if (row >= this.rows_.size()) {
			return;
		}
		this.getRowInternal(row).setComponent(column, component);
		update();
	}

	private RowPanel getRowInternal(int row) {
		if (row >= this.rows_.size()) {
			return null;
		}
		return this.rows_.get(row);
	}
	
	
	/** 
	 * Override super methods for safety reasons 
	 * 
	 */
	@Override
	public Component add(Component arg0, int arg1) {
		// TODO Auto-generated method stub
		return arg0;
	}

	@Override
	public void add(Component arg0, Object arg1, int arg2) {
		
	}

	@Override
	public void add(Component arg0, Object arg1) {
		
	}

	@Override
	public Component add(Component arg0) {
		return super.add(arg0);
	}

	@Override
	public Component add(String arg0, Component arg1) {
		return arg1;
	}

	@Override
	public int getComponentCount() {
		return this.componentCount();
	}

	@Override
	public Component[] getComponents() {
		Component comps[] = new Component[this.componentCount()];
		this.rows_.toArray(comps);
		return comps;
	}

	private int componentCount() {
		return this.rowCount();
	}
	
	
}
