package jagafa.util.view;

import java.awt.GridLayout;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JPanel;

public class RowPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -3015307949913335671L;

	private int columns_;

	private List<JComponent> components_;

	public RowPanel(int columns) {
		super();
		this.columns_ = columns;
		this.components_ = new LinkedList<JComponent>();

		for (int i = 0; i < columns; i++) {
			this.components_.add(null);
		}
	}

	/**
	 * Get the number of columns in the row
	 * @return
	 */
	public int columnCount() {
		return this.columns_;
	}

	/**
	 * Get the component at the n-th column of the row
	 * @return The component at column col. 
	 * 		   If it is not set (=null), return an empty JPanel object 
	 */
	public JComponent getComponent(int col) {
		if (col >= this.columns_) {
			return null;
		}
		JComponent comp = this.components_.get(col);
		if (comp == null) {
			return new JPanel();
		}
		return comp;
	}

	/**
	 * Set the component at column i to the component specified
	 * @param column
	 * @param component
	 */
	public void setComponent(int column, JComponent component) {
		if (column >= this.columns_) {
			return;
		}
		this.components_.set(column, component);
		update();
	}

	/**
	 * Adds a component to the next free column in the row
	 * @param component
	 */
	public void addComponent(JComponent component) {

		for (int i = 0; i < this.columns_; i++) {
			if (this.components_.get(i) == null) {
				this.setComponent(i, component);

				return;
			}
		}

	}

	/**
	 * Remove the component at the column specified
	 * @param column
	 */
	public void removeComponent(int column) {
		if (column >= this.columns_) {
			return;
		}
		this.components_.set(column, null);
		update();
	}

	/**
	 * Internally update the RowPanel. Called after every change 
	 *
	 */
	private void update() {
		
		this.removeAll();

		this.setLayout(new GridLayout(1, this.columnCount()));

		for (int j = 0; j < this.columnCount(); j++) {
			JComponent comp = this.getComponent(j);
			this.add(comp);
		}

	}
}
