/**
 * 
 */
package jagafa.view;

import jagafa.naming.CardNames;
import jagafa.object.Board;
import jagafa.object.Card;
import jagafa.object.Player;

import java.awt.Color;
import java.awt.GridLayout;
import java.util.LinkedList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

/**
 * A Panel with four CardButton objects that represent the current active Board
 * 
 */
public class BoardPanel extends JPanel {

	private static final int BOARD_SIZE = 4;

	private List<CardButton> buttonList_;

	private static final long serialVersionUID = -7141846003079501463L;

	/**
	 * Constructor: Set Layout and init CardButtons
	 */
	public BoardPanel() {
		this.setLayout(new GridLayout(2, 2));
		this.buttonList_ = new LinkedList<CardButton>();
		this.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		
		for (int i = 0; i < BOARD_SIZE; i++) {

			CardButton button = new CardButton(null, CardButton.HORIZONTAL);
			this.add(button);
			this.buttonList_.add(button);
		}

		this.setEnabled(false);
		this.setFocusable(false);

	}

	/**
	 * Reset the CardButtons to the buttons in the Board given
	 */
	public void updateBoardUI(Board board) {

		clearCards();

		for (int i = 0; i < BOARD_SIZE; i++) {
			Player play = board.getPlayerOfPos(i);
			int playerNr = i;
			if (play != null) {
				playerNr = Integer.valueOf(play.getName()).intValue();
				Card c = board.get(i);

				this.getButton(getPosFor(playerNr)).setCardName(CardNames.getName(c));
			}
		}

	}

	/**
	 * Sets all Card names (reps. images) to null
	 */
	private void clearCards() {
		for (int i = 0; i < BOARD_SIZE; i++) {
			this.getButton(i).setCardName(null);
		}
	}

	/**
	 * Returns the button corresponding to the player with the number given
	 */
	private int getPosFor(int playerNr) {
		switch (playerNr) {
		case 0:
			return 0;
		case 1:
			return 2;
		case 2:
			return 3;
		case 3:
			return 1;
		}
		
		
		
		return 0;
	}

	/**
	 * Returns the button i in the button list
	 */
	private CardButton getButton(int i) {
		return this.buttonList_.get(i);
	}
}
