/*
 * Created on 25.02.2005 Filename: JGameCard.java
 */
package jagafa.view;

import jagafa.flags.TestingFlags;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JButton;

/**
 * A JButton with a Card image
 */
public class CardButton extends JButton {

	private static final long serialVersionUID = -7302102071467457143L;

	private ImageIcon icon_;
	
	private int orientation_ = 0;

	/**
	 * Constructor: Create a new CardButton with the Card specified (as a string) and
	 * the card orientation (HORIZONTAL or VERTICAL)
	 */
	public CardButton(String cardName, int orientation) {
		super();

		if (cardName != null) {
			icon_ = this.createImageIcon(TestingFlags.RESOURCE_DIR + cardName
					+ TestingFlags.IMAGE_EXTENSION);
			this.setVisible(true);
		} else {
			icon_ = this.createImageIcon(TestingFlags.RESOURCE_DIR + TestingFlags.EMPTY_IMAGE);
			this.setVisible(false);
		}
		this.setContentAreaFilled(false);
		this.setDefaultCapable(false);
		this.setFocusPainted(false);
		this.setMargin(new Insets(0, 0, 0, 0));
		this.setFocusable(false);
		this.setOpaque(true);
		this.setSize(100, 50);
		//this.setBorderPainted(false);

		this.orientation_ = orientation;
		this.validate();
	}

	/**
	 * Set a new cardname and reinitialize the card image
	 */
	public void setCardName(String name) {
		if (name != null) {
			icon_ = this.createImageIcon(TestingFlags.RESOURCE_DIR + name
					+ TestingFlags.IMAGE_EXTENSION);
			this.setVisible(true);
		} else {

			icon_ = this.createImageIcon(TestingFlags.RESOURCE_DIR + TestingFlags.EMPTY_IMAGE);
			this.setVisible(false);
		}

		this.validate();
	}

	/**
	 * Paints the component in VERTICAL or HORIZONTAL orientation
	 */
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);

		Graphics2D componentG = (Graphics2D) g;

		int iconWidth = this.icon_.getIconWidth();
		int iconHeight = this.icon_.getIconHeight();

		/* Turn the icon into a more convenient image format */
		BufferedImage rawImage = new BufferedImage(iconWidth, iconHeight,
				BufferedImage.TYPE_INT_RGB);

		Graphics2D imageG2D = (Graphics2D) rawImage.getGraphics();
		this.icon_.paintIcon(this, imageG2D, 0, 0);

		if (orientation_ == VERTICAL) {
			double xscale = (double) this.getWidth() / (double) rawImage.getHeight(this);
			double yscale = (double) this.getHeight() / (double) rawImage.getWidth(this);

			/* This image holds the rotated and translated image in original size */
			BufferedImage rotatedImage = new BufferedImage(iconHeight, iconWidth,
					BufferedImage.TYPE_INT_RGB);
			Graphics2D rotatedG = (Graphics2D) rotatedImage.getGraphics();

			/* Rotate the image by 90 degrees and translate */
			AffineTransform trans = AffineTransform.getRotateInstance(Math.toRadians(90));
			trans.concatenate(AffineTransform.getTranslateInstance(0, -iconHeight));

			rotatedG.drawImage(rawImage, trans, this);

			/* Scale to fit on the component */
			AffineTransform trans2 = AffineTransform.getScaleInstance(xscale, yscale);

			/* Transform and then draw the final image onto the component */
			componentG.drawImage(rotatedImage, trans2, this);

		} else {
			double xscale = (double) this.getWidth() / (double) rawImage.getWidth(this);
			double yscale = (double) this.getHeight() / (double) rawImage.getHeight(this);

			/* Just scale the image */
			AffineTransform trans = AffineTransform.getScaleInstance(xscale, yscale);
			componentG.drawImage(rawImage, trans, this);
		}
		this.validate();
	}

	/**
	 * Load and Create the ImageIcon from the card image file specified
	 * by path (Format: CardColor CardValue.jpg)
	 */
	private ImageIcon createImageIcon(String path) {
		URL imgURL = this.getClass().getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			System.err.println("Image nicht gefunden: " + path);
			return null;
		}
	}

}