/**
 * 
 */
package jagafa.view;

import jagafa.flags.TestingFlags;
import jagafa.naming.CardNames;
import jagafa.object.Card;
import jagafa.object.Hand;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JLayeredPane;
import javax.swing.event.EventListenerList;

/**
 * HandPanel is a Panel holding the CardButtons of a Players Hand
 * 
 */
public class HandPanel extends JLayeredPane implements ActionListener{

	private static final int COLORS = 4;

	private static final int HAND_SIZE = 9;

	private List<CardButton> cardButtons_;

	private static final long serialVersionUID = 5051908824891500716L;

	public static final int HORIZONTAL = 0;

	public static final int VERTICAL = 1;

	/**
	 * Constructor: Create a new HandPanel with the orientation given
	 * orientation = HORIZONTAL or VERTICAL
	 */
	public HandPanel(int orientation, int cardHeight) {
		super();
		this.listeners_ = new EventListenerList();
		this.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		if (orientation == HORIZONTAL) {
			//this.setLayout(new GridLayout(1, HAND_SIZE+3));
			//this.setLayout(new OverlayLayout(this));
		} else {
			//this.setLayout(new GridLayout(HAND_SIZE, 1));
			//this.setLayout(new OverlayLayout(this));
		}

		this.cardButtons_ = new LinkedList<CardButton>();
		this.setPreferredSize(new Dimension(600,200));
		int height = TestingFlags.CARD_HEIGHT;
		double ratio = TestingFlags.CARD_RATIO;
		for (int i = 0; i < 9; i++) {
			CardButton button;
		
			if (orientation == HORIZONTAL) {
				Card c = new Card((int) (Math.random() * COLORS), (int) (Math
						.random() * HAND_SIZE));
				button = new CardButton(CardNames.getName(c), HORIZONTAL);
				
				
				button.setBounds(i*50,0,(int)(height*ratio),height);
				
			} else {
				Card c = new Card((int) (Math.random() * COLORS), (int) (Math
						.random() * HAND_SIZE));

				button = new CardButton(CardNames.getName(c), VERTICAL);
				button.setBounds(0,i*50,height,(int)(height*ratio));
				
			}
			
			button.setName(String.valueOf(i));
			this.cardButtons_.add(button);
			this.add(button,new Integer(i),1);

		}
		this.validate();
	}

	/**
	 * Update the panel with the cards in the hand given
	 */
	public void updateHandUI(Hand h) {
		for (int i = 0; i < h.size(); i++) {
			this.getButton(i).setCardName(h.get(i).toString());
			this.getButton(i).validate();
		}
		for (int j = 0;j<HAND_SIZE-h.size();j++){
			this.getButton(h.size()+j).setCardName(null);
		}
		this.validate();
		
	}

	/**
	 * Returns CardButton i in the cardButtonList
	 */
	private CardButton getButton(int i) {
		return this.cardButtons_.get(i);
	}

	private EventListenerList listeners_;
	
	
	/** 
	 * Add Listeners to all Cards in the Hand 
	 */
	public void addHandListener(ActionListener l) {
		listeners_.add(ActionListener.class,l);
		for (int buttonNr = 0; buttonNr < HAND_SIZE; buttonNr++) {
			this.getButton(buttonNr).addActionListener(this);
		}
	}

	
	/** 
	 * Informs the Container (eg. JassFrame) of a buttonPressed event
	 */
	private void cardPressed(CardButton source){
		Object[] listeners = this.listeners_.getListenerList();
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == ActionListener.class) {
				ActionEvent fooEvent = new ActionEvent(this, 0, source.getName());
				((ActionListener) listeners[i + 1]).actionPerformed(fooEvent);
			}
		}
		this.validate();
	}

	/**
	 *  Called when a cardButton has been pressed
	 */
	public void actionPerformed(ActionEvent e) {
		CardButton pressedCard = (CardButton) e.getSource();
		
		cardPressed(pressedCard);
	}


}
