/**
 * 
 */
package jagafa.view.newui;

import jagafa.flags.TestingFlags;
import jagafa.object.Hand;
import jagafa.util.view.AbsConstr;
import jagafa.util.view.AbsoluteLayout;
import jagafa.view.BoardPanel;
import jagafa.view.HandPanel;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.event.EventListenerList;

/**
 * JassFrame is a Frame holding the four HandPanel and the BoardPanel Panels 
 */
public class JassRoundPanel extends JPanel implements ActionListener {

	private static final int PANEL_WIDTH = 700;

	private static final int PANEL_HEIGHT = 580;

	private static final int PLAYERS = 4;

	private static final long serialVersionUID = 4602876072322144533L;

	private static final int VERT_HANDY = (int)(TestingFlags.CARD_HEIGHT*TestingFlags.CARD_RATIO)+10;//70;

	private static final int HORIZ_HANDX = 140;

	private static final int VERT_HANDHEIGHT = (int)(10.0*TestingFlags.CARD_OVERLAP);//492;

	private static final int VERT_HANDWIDTH = TestingFlags.CARD_HEIGHT; // 100

	private static final int HORIZ_HANDHEIGHT = VERT_HANDWIDTH; // 100

	private static final int HORIZ_HANDWIDTH = VERT_HANDHEIGHT;

	private List<HandPanel> handPanels_;

	private HandPanel handPanel1_;

	private HandPanel handPanel2_;

	private HandPanel handPanel3_;

	private HandPanel handPanel4_;

	private BoardPanel boardPanel_;

	private EventListenerList listeners_ = new EventListenerList();

	public JassRoundPanel() {
		this.handPanels_ = new LinkedList<HandPanel>();

		initComponents();

		/* TODO: Add automatic hand hiding on computer players */
		if (TestingFlags.showAllHands_) {

		} else {
			hideHand(1);
			hideHand(2);
			hideHand(3);
			hideHand(0);
		}

		installHandListeners();

		this.setPreferredSize(new Dimension(PANEL_WIDTH, PANEL_HEIGHT));
	}

	public void hideHand(int i) {
		this.handPanels_.get(i).setVisible(false);
	}

	public void showHand(int i) {
		this.handPanels_.get(i).setVisible(true);
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	private void initComponents() {
		handPanel1_ = new HandPanel(HandPanel.HORIZONTAL, TestingFlags.CARD_HEIGHT);
		handPanel2_ = new HandPanel(HandPanel.VERTICAL, TestingFlags.CARD_HEIGHT);
		handPanel3_ = new HandPanel(HandPanel.HORIZONTAL, TestingFlags.CARD_HEIGHT);
		handPanel4_ = new HandPanel(HandPanel.VERTICAL, TestingFlags.CARD_HEIGHT);
		boardPanel_ = new BoardPanel();

		this.setLayout(new AbsoluteLayout());

		createHandPanels();

	}

	/**
	 * 
	 */
	private void createHandPanels() {
		this.add(handPanel1_, new AbsConstr(HORIZ_HANDX, 0, HORIZ_HANDWIDTH,
				HORIZ_HANDHEIGHT));

		handPanel1_.setName(String.valueOf(0));
		handPanels_.add(handPanel1_);

		this.add(handPanel2_, new AbsConstr(0, VERT_HANDY, VERT_HANDWIDTH,
				VERT_HANDHEIGHT));
		handPanels_.add(handPanel2_);
		handPanel2_.setName(String.valueOf(1));

		this.add(handPanel3_, new AbsConstr(HORIZ_HANDX, 550, HORIZ_HANDWIDTH,
				HORIZ_HANDHEIGHT));
		handPanels_.add(handPanel3_);
		handPanel3_.setName(String.valueOf(2));

		this.add(handPanel4_, new AbsConstr(690, VERT_HANDY, VERT_HANDWIDTH,
				VERT_HANDHEIGHT));
		handPanels_.add(handPanel4_);
		handPanel4_.setName(String.valueOf(3));

		boardPanel_.setLayout(new java.awt.GridLayout(2, 2));

		int boardPanelHeight = 300;
		this.add(boardPanel_, new AbsConstr(300, 220,
				(int) (TestingFlags.CARD_RATIO * boardPanelHeight), boardPanelHeight));
	}

	// Install the listeners for the hands
	private void installHandListeners() {

		for (int i = 0; i < PLAYERS; i++) {
			HandPanel hPanel = this.getHandPanel(i);
			hPanel.addHandListener(this);
		}
	}

	/*
	 * Is called by the handPanel's cardPressed() method
	 */
	public void actionPerformed(ActionEvent e) {
		String button = e.getActionCommand();
		String hand = ((HandPanel) e.getSource()).getName();

		this.cardPressed(hand + button);

	}

	// Installs the top card-listeners (Controller)
	public void addCardListener(ActionListener l) {
		this.listeners_.add(ActionListener.class, l);
	}

	/** 
	 * Informs the top Listener (Controller) that a cardButton has been pressed
	 * MESSAGE contains the hand- and button numbers of the cardButton
	 */
	private void cardPressed(String message) {
		Object[] listeners = this.listeners_.getListenerList();
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == ActionListener.class) {
				ActionEvent fooEvent = new ActionEvent(this, 0, message);
				((ActionListener) listeners[i + 1]).actionPerformed(fooEvent);
			}
		}
	}

	/**
	 * Returns the HandPanel i
	 * ??? i = 0 --> top, i= 1 --> left, i = 2 --> bottom, i=3 --> right ??? 
	 */
	private HandPanel getHandPanel(int i) {
		return this.handPanels_.get(i);
	}

	/**
	 * Returns the BoardPanel object
	 */
	public BoardPanel getBoardPanel() {
		return this.boardPanel_;
	}

	public void updateHandPanel(Hand hand, int i) {
		this.getHandPanel(i).updateHandUI(hand);

	}

}
