/**
 * 
 */
package jagafa.view.newui;

import jagafa.JassRound;
import jagafa.naming.ESwissColorNames;
import jagafa.naming.ESwissValueNames;
import jagafa.object.Card;
import jagafa.object.CardList;
import jagafa.object.Hand;
import jagafa.object.Player;
import jagafa.stats.PlayerStats;
import jagafa.stats.RoundStats;

import java.awt.Color;
import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.Border;

/**
 * StatView is a Frame displaying some game information including team scores,
 * Bock lists and gone cards of every color
 * 
 */
public class RoundStatsPanel extends JPanel {

	private static final int COLORS = 4;

	private static final int STATS_YPOS = 410;

	private static final int STATS_HEIGHT = 325;

	private static final int STATS_WIDTH = 300;

	private static final long serialVersionUID = -5864758237368176044L;

	private JassRound round_;

	private JPanel gonePanel_;

	private JPanel scorePanel_;

	private JPanel colorPanel_;

	private JPanel bockPanel_;

	private PlayerStats playerStats_;

	private RoundStats roundStats_;

	public RoundStatsPanel(JassRound round) {
		this.round_ = round;
		this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

		this.setSize(STATS_WIDTH, STATS_HEIGHT);
		this.setLocation(0, STATS_YPOS);
		addTitleBorder("Stats:");

		this.playerStats_ = new PlayerStats(this.round_,this.round_.activePlayer());
		this.roundStats_ = new RoundStats(this.round_);
		
		if (this.round_ != null) {
			createScorePanel();
		//	createGonePanel();
			createBockPanel();
			createColorPanel();
		}
	}

	private void addTitleBorder(String title) {
		Border rawBorder = BorderFactory.createLineBorder(Color.BLACK);

		Border titledB = BorderFactory.createTitledBorder(rawBorder, title);
		this.setBorder(titledB);

	}

	/* (non-Javadoc)
	 * @see jagafa.view.newui.RoundStats#update()
	 */
	public void update() {
		if (this.round_.getRules() == null) {
			return;
		}

		this.playerStats_.update();
		this.roundStats_.update();

		//this.updateGonePanel();
		this.updateBockPanel();
		this.updateScorePanel();
		this.updateColorPanel();
		this.validate();
	}

	/**
	 * 
	 */
	private void createColorPanel() {
		colorPanel_ = new JPanel();
		colorPanel_.setName("Colors in Hand:");
		colorPanel_.setLayout(new GridLayout(2, 2));

		String labelText = "";
		for (int i = 0; i < COLORS; i++) {
			CardList colorC = this.playerStats_.getColorList()[i];

			labelText += ESwissColorNames.getName(i) + " : " + colorC.size() + ", ";
		}

		JLabel label = new JLabel(labelText);
		colorPanel_.add(label);
		this.add(colorPanel_);

		Border border = BorderFactory.createTitledBorder(BorderFactory
				.createLineBorder(Color.BLACK), "Colors in Hand:");
		this.colorPanel_.setBorder(border);

	}

	private void updateColorPanel() {
		this.colorPanel_.removeAll();

		String labelText = "";
		for (int i = 0; i < COLORS; i++) {
			CardList colorC = this.playerStats_.getColorList()[i];

			labelText += ESwissColorNames.getName(i) + " : " + colorC.size() + ", ";
		}

		JLabel label = new JLabel(labelText);
		colorPanel_.add(label);

	}

	/**
	 * 
	 */
	private void createScorePanel() {
		scorePanel_ = new JPanel();
		scorePanel_.setName("Scores:");
		Border border = BorderFactory.createTitledBorder(BorderFactory
				.createLineBorder(Color.BLACK), "Scores:");
		this.scorePanel_.setBorder(border);

		scorePanel_.setLayout(new GridLayout(1, 2));
		int sum1 = this.roundStats_.getScoreTeam1();
		int sum2 = this.roundStats_.getScoreTeam2();
		JLabel team1 = new JLabel("Player 0 and 2: " + sum1);
		JLabel team2 = new JLabel("Player 1 and 3: " + sum2);
		scorePanel_.add(team1);
		scorePanel_.add(team2);
		this.add(scorePanel_);

	}

	/**
	 * 
	 */
	private void createGonePanel() {

		gonePanel_ = new JPanel();
		gonePanel_.setName("Gone Cards:");
		Border border = BorderFactory.createTitledBorder(BorderFactory
				.createLineBorder(Color.BLACK), "Gone Cards:");
		this.gonePanel_.setBorder(border);
		gonePanel_.setLayout(new GridLayout(1, COLORS));

		this.add(new JScrollPane(gonePanel_));
	}

	private void updateGonePanel() {
		this.gonePanel_.removeAll();
		for (int i = 0; i < COLORS; i++) {

			CardList goneofcol = this.roundStats_.getGoneList()[i];

			Hand h = new Hand(goneofcol);
			h.sort(this.round_.getRules());
			goneofcol = h;

			String labelText = ESwissColorNames.getName(i) + " : " + goneofcol.size()
					+ "  ";
			for (int j = 0; j < goneofcol.size(); j++) {
				Card c = goneofcol.get(j);

				labelText += " " + ESwissValueNames.getName(c.getValue());
			}

			JLabel label = new JLabel(labelText);
			gonePanel_.add(label);

		}

		this.validate();
	}

	/**
	 * 
	 */
	private void updateBockPanel() {
		this.bockPanel_.removeAll();

		if (round_.getRules().specialColor() != -1) {
			String trumpfe = "Anzahl Trmpfe: ";
			CardList trumpList = this.playerStats_.getTrumpfList();

			int trumpfNo = trumpList.size();

			trumpfe += trumpfNo;
			JLabel trumpfLabel = new JLabel(trumpfe);
			this.bockPanel_.add(trumpfLabel);
		}

		for (int i = 0; i < 9; i++) {
			String labelText = new String();
			labelText += i + ". Bock: ";

			JLabel bockLabel = new JLabel();
			CardList bockNth = this.playerStats_.getBockList()[i];
			for (int j = 0; j < bockNth.size(); j++) {
				labelText += bockNth.get(j).toString() + ", ";

			}
			bockLabel.setText(labelText);
			bockPanel_.add(bockLabel);
		}

		bockPanel_.validate();
		this.validate();
	}

	private void createBockPanel() {
		bockPanel_ = new JPanel();
		bockPanel_.setName("Bock Cards:");

		bockPanel_.setLayout(new GridLayout(9, 1));
		this.add(new JScrollPane(bockPanel_));
		Border border = BorderFactory.createTitledBorder(BorderFactory
				.createLineBorder(Color.BLACK), "Bock Cards:");
		this.bockPanel_.setBorder(border);
	}

	private void updateScorePanel() {
		scorePanel_.removeAll();
		scorePanel_.setLayout(new GridLayout(2, 1));
		int sum1 = this.roundStats_.getScoreTeam1();
		int sum2 = this.roundStats_.getScoreTeam2();
		JLabel team1 = new JLabel("Player 0 and 2: " + sum1);
		JLabel team2 = new JLabel("Player 1 and 3: " + sum2);
		scorePanel_.add(team1);
		scorePanel_.add(team2);
	}

	/* (non-Javadoc)
	 * @see jagafa.view.newui.RoundStats#setRound(jagafa.JassRound)
	 */
	public void setRound(JassRound round) {
		this.round_ = round;

	}

	public JComponent getView() {
		return this;
	}

	public void update(Player player) {
		this.playerStats_.setPlayer(player);
		update();
		
	}
}
