/**
 * 
 */
package jagafa.view.newui;

import jagafa.JassRound;
import jagafa.ai.core.RuleSetChoiceAI;
import jagafa.object.Player;
import jagafa.rule.RuleSet;
import jagafa.scores.ScoreTable;
import jagafa.util.view.TimedDialog;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import javax.swing.AbstractButton;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.Border;

/**
 * @author Besitzer
 *
 */
public class RuleChoicePanel extends JPanel implements ActionListener {
	private static final String SCHIEBEN = "Hand choice over";

	private static final String LET_AI_CHOOSE = "Let AI choose";

	private static final String DEAL_CARDS = "Deal";

	private static final String START_ROUND = "Set Rule";

	private static final String RESET_SCORETABLE = "Reset scores";

	private static final int RCHOICE_YPOS = 25;

	private static final int RCHOICE_WIDTH = 140;

	private static final int ADDITIONAL_CONTROLS = 6;

	private static final long serialVersionUID = 2918074812070652105L;

	private static final String TITLE = "Rule choice";

	private ButtonGroup group_;

	private int selectedRule_;

	private JassRound round_;

	private ScoreTable scoreTable_;

	private List<RuleSet> rules_;

	private JLabel ruleSetLabel_;

	private JComponent roundUI_;

	public RuleChoicePanel(JassRound round, JComponent roundUI) {
		this.roundUI_ = null;
		this.round_ = round;
		this.scoreTable_ = round.getScoreTable();

		this.rules_ = scoreTable_.getRules();

		initControls();

		this.setLocation(0, RCHOICE_YPOS);
		this.setPreferredSize(new Dimension(RCHOICE_WIDTH, this.group_.getButtonCount()
				* 15 + ADDITIONAL_CONTROLS * 20));

	}

	public void setRound(JassRound round) {
		this.round_ = round;
		this.scoreTable_ = round.getScoreTable();
		this.rules_ = this.scoreTable_.getRules();
	}

	private void initControls() {
		this.addTitleBorder(TITLE);
		GridLayout layout = new GridLayout(rules_.size() + ADDITIONAL_CONTROLS, 1);
		this.setLayout(layout);

		this.group_ = new ButtonGroup();

		Iterator<RuleSet> ruleIter = rules_.iterator();
		while (ruleIter.hasNext()) {
			createOption(ruleIter.next());
		}

		this.group_.getElements().nextElement().setSelected(true);
		ruleSetLabel_ = new JLabel("Trump: " + this.round_.getRules());

		JButton okButton = new JButton(START_ROUND);
		JButton dealButton = new JButton(DEAL_CARDS);
		JButton aicButton = new JButton(LET_AI_CHOOSE);
		JButton resButton = new JButton(RESET_SCORETABLE);
		JButton schiebeButton = new JButton(SCHIEBEN);

		this.add(ruleSetLabel_);
		this.add(okButton);
		this.add(dealButton);
		this.add(aicButton);
		this.add(resButton);
		this.add(schiebeButton);

		okButton.addActionListener(this);
		aicButton.addActionListener(this);
		dealButton.addActionListener(this);
		resButton.addActionListener(this);
		schiebeButton.addActionListener(this);
	}

	private void createOption(RuleSet rule) {
		int multiplier = this.scoreTable_.getMultiplier(rule);

		JRadioButton ruleRadio = new JRadioButton(rule.getName() + " " + multiplier + "x");
		ruleRadio.setName(String.valueOf(group_.getButtonCount()));
		this.add(ruleRadio);

		this.group_.add(ruleRadio);
	}

	private int getSelectedIndex() {
		Enumeration<AbstractButton> elements = this.group_.getElements();
		int selIndex = 0;
		while (elements.hasMoreElements()) {
			AbstractButton button = elements.nextElement();
			if (button.isSelected()) {
				selectedRule_ = selIndex;
				break;
			}
			selIndex++;
		}
		return selectedRule_;
	}

	public void actionPerformed(ActionEvent e) {
		/* TODO: RuleChoicePanel should only be UI and not start any Rounds!!! */

		if (!this.round_.isRoundOver() && this.round_.isRunning()) {

			//System.out.println("Running!");
			new TimedDialog("Game is already running!",roundUI_);
			return;
		}

		String command = e.getActionCommand();

		if (command.equals(START_ROUND)) {
			startRoundHumanChoice();
		} else if (command.equals(DEAL_CARDS)) {
			dealCards();
			this.startRoundComputerChoice();
		} else if (command.equals(LET_AI_CHOOSE)) {
			startRoundComputerChoice();

		} else if (command.equals(RESET_SCORETABLE)) {
			resetScores();
		} else if (command.equals(SCHIEBEN)) {
			if (!this.scoreTable_.changeChoosingPlayer()) {

				/*System.out
				 .println("Es kann nur einmal geschoben werden! Bitte Trumpf bestimmen!");*/
				//System.out.println("You can only hand over once! Please choose!");
				new TimedDialog("You can only hand over once! Please choose!", roundUI_);
				
			} else {
				printAIHint();
				this.startRoundComputerChoice();
			}
		}

		if (this.round_.getRules() != null) {
			this.ruleSetLabel_.setText("Current: " + this.round_.getRules().getName());
		
		}
	}

	public void updateRuleLabel() {
		if (this.round_.getRules() != null) {
			this.ruleSetLabel_.setText("Current: " + this.round_.getRules().getName());
		}
	}

	/**
	 * Reset scoretable
	 *
	 */
	private void resetScores() {
		this.scoreTable_.resetScores();
		this.round_.reset();
		new TimedDialog("Scores Reset to 0!",roundUI_);
		this.dealCards();
	}

	/**
	 * Start the round with AI chosen RuleSet
	 *
	 */
	private void startRoundComputerChoice() {
		Player startPlayer = this.round_.getPlayer(this.scoreTable_.getStartPlayer());
		Player choosingPlayer = this.round_.getPlayer(this.scoreTable_
				.getChoosingPlayer());

		if (choosingPlayer.isComputer()) {
			RuleSet choice = choosingPlayer.getAI().chooseRuleSet();

			if (choice == null) {
				this.scoreTable_.changeChoosingPlayer();
				choosingPlayer = this.round_.getPlayer(this.scoreTable_
						.getChoosingPlayer());
				if (!choosingPlayer.isComputer()) {
					new TimedDialog("Your Partner handed the choice over to you! Please choose a Trump!",roundUI_);
					return;
				}
				
				startPlayer = this.round_.getPlayer(this.scoreTable_.getStartPlayer());

				choice = choosingPlayer.getAI().chooseRuleSet();
			}

			this.round_.setRules(choice);
			new TimedDialog("Player " + choosingPlayer + " says : " + choice.getName()
					+ "\n",roundUI_);

			/*System.out.println("Player " + choosingPlayer + " says : " + choice.getName()
					+ "\n");
			*/
			this.round_.start(startPlayer);

			this.round_.toggleNotify();
		} else {
			//System.out.println("Spieler ist kein Computer! Bitte Trumpf bestimmen!");
			//System.out.println("Please choose a Trump!");
			new TimedDialog("Please choose a Trump!",roundUI_);
			
			
			
		}
	}

	/**
	 * Deal the cards
	 *
	 */
	private void dealCards() {
		this.round_.setRules(this.rules_.get(getSelectedIndex()));
		this.round_.reset();
		Player startPlayer = this.round_.getPlayer(this.scoreTable_.getStartPlayer());

		

		

		this.round_.toggleNotify();
		printAIHint();

	}

	private void printAIHint() {
		Player startPlayer = this.round_.getPlayer(this.scoreTable_.getStartPlayer());
		Player choosingPlayer = this.round_.getPlayer(this.scoreTable_
				.getChoosingPlayer());

		RuleSet choice = RuleSetChoiceAI.chooseRuleSet(choosingPlayer, this.round_);
		this.round_.setRules(choice);

		if (!choosingPlayer.isComputer()) {
		if (this.round_.getRules() != null) {
			//Dump.displayMessage("AI hint:\t " + choice.getName(),this);
			new TimedDialog("AI hint: " + choice.getName(),roundUI_);
			//System.out.println("AI hint: " + choice.getName());
			
		} else {
			//Dump.displayMessage("AI hint:\t Hand over!",this);
			new TimedDialog("AI hint: Hand over!\n",roundUI_);
			//System.out.println("AI hint: Hand over!");
		}}
	}

	/**
	 * Start the round with the user-selected RuleSet
	 *
	 */
	private void startRoundHumanChoice() {

		Player startPlayer = this.round_.getPlayer(this.scoreTable_.getStartPlayer());
		Player choosingPlayer = this.round_.getPlayer(this.scoreTable_
				.getChoosingPlayer());

		if (choosingPlayer.isComputer()) {
			this.startRoundComputerChoice();
		} else {
			this.round_.setRules(this.rules_.get(getSelectedIndex()));
			//System.out.println("Trump: " + this.rules_.get(getSelectedIndex()).getName());
			new TimedDialog("Trump is now: "
					+ this.rules_.get(getSelectedIndex()).getName(),roundUI_);
			this.round_.start(startPlayer);
			this.round_.toggleNotify();
		}
	}

	private void addTitleBorder(String string) {
		Border b = BorderFactory.createLineBorder(Color.BLACK);

		Border titleB = BorderFactory.createTitledBorder(b, string);
		this.setBorder(titleB);

	}

	public void setRoundUI(JFrame frame) {
		this.roundUI_ = frame.getRootPane();
		
	}

	

	
}
